@@ -47,6 +47,7 @@
 #include "force.h"
 #include "pair.h"
 #include "pair_gran.h"
+#include "pair_gran_proxy.h"
 #include "modify.h"
 #include "neighbor.h"
 #include "neigh_request.h"
@@ -66,10 +67,10 @@ using namespace LAMMPS_NS;
 
 /* ---------------------------------------------------------------------- */
 
-ComputePairGranLocal::ComputePairGranLocal(LAMMPS *lmp, int narg, char **arg) :
-  Compute(lmp, narg, arg)
+ComputePairGranLocal::ComputePairGranLocal(LAMMPS *lmp, int &iarg, int narg, char **arg) :
+  Compute(lmp, iarg, narg, arg)
 {
-  if (narg < 3) error->all(FLERR,"Illegal compute pair/gran/local or wall/gran/local command");
+  if (narg < iarg) error->all(FLERR,"Illegal compute pair/gran/local or wall/gran/local command");
 
   local_flag = 1;
   nmax = 0;
@@ -79,16 +80,16 @@ ComputePairGranLocal::ComputePairGranLocal(LAMMPS *lmp, int narg, char **arg) :
   posflag = velflag = idflag = fflag = torqueflag = histflag = areaflag = 1;
 
   // do not store fn, ft, heat flux, delta by default
-  fnflag = ftflag  = torquenflag = torquetflag = deltaflag = heatflag = 0;
+  fnflag = ftflag  = torquenflag = torquetflag = deltaflag = heatflag = cpflag = 0;
 
   //no extra distance for building the list of pairs
   verbose = false;
 
   // if further args, store only the properties that are listed
-  if(narg > 3)
-     posflag = velflag = idflag = fflag = fnflag = ftflag = torqueflag = torquenflag = torquetflag = histflag = areaflag = deltaflag = heatflag = 0;
+  if(narg > iarg)
+     posflag = velflag = idflag = fflag = fnflag = ftflag = torqueflag = torquenflag = torquetflag = histflag = areaflag = deltaflag = heatflag = cpflag  = 0;
 
-  for (int iarg = 3; iarg < narg; iarg++)
+  for (; iarg < narg; iarg++)
   {
     //int i = iarg-3;
     if (strcmp(arg[iarg],"pos") == 0) posflag = 1;
@@ -104,6 +105,7 @@ ComputePairGranLocal::ComputePairGranLocal(LAMMPS *lmp, int narg, char **arg) :
     else if (strcmp(arg[iarg],"contactArea") == 0) areaflag = 1;
     else if (strcmp(arg[iarg],"delta") == 0) deltaflag = 1;
     else if (strcmp(arg[iarg],"heatFlux") == 0) heatflag = 1;
+    else if (strcmp(arg[iarg],"contactPoint") == 0) cpflag = 1;
     else if (strcmp(arg[iarg],"verbose") == 0) verbose = true;
     else if (strcmp(arg[iarg],"extraSurfDistance") == 0) error->all(FLERR,"this keyword is deprecated; neighbor->contactDistanceFactor is now used directly");
     else if(0 == strcmp(style,"wall/gran/local") || 0 == strcmp(style,"pair/gran/local"))
@@ -270,7 +272,7 @@ void ComputePairGranLocal::init_cpgl(bool requestflag)
   if(histflag && dnum == 0) error->all(FLERR,"Compute pair/gran/local or wall/gran/local can not calculate history values since pair or wall style does not compute them");
   // standard values: pos1,pos2,id1,id2,extra id for mesh wall,force,torque,contact area
 
-  nvalues = posflag*6 + velflag*6 + idflag*3 + fflag*3 + fnflag*3 + ftflag*3 + torqueflag*3 + torquenflag*3 + torquetflag*3 + histflag*dnum + areaflag + deltaflag + heatflag;
+  nvalues = posflag*6 + velflag*6 + idflag*3 + fflag*3 + fnflag*3 + ftflag*3 + torqueflag*3 + torquenflag*3 + torquetflag*3 + histflag*dnum + areaflag + deltaflag + heatflag + cpflag*3;
   size_local_cols = nvalues;
 
 }
@@ -556,11 +558,39 @@ void ComputePairGranLocal::add_pair(int i,int j,double fx,double fy,double fz,do
     }
     if(deltaflag)
     {
+        if(atom->superquadric_flag)
+        {
+          int alpha1_offset = get_history_offset("alpha1_offset");
+          int alpha2_offset = get_history_offset("alpha2_offset");
+          array[ipair][n++] = hist[alpha1_offset]+hist[alpha2_offset];
+        } else {
+          radi = atom->radius[i];
+          radj = atom->radius[j];
+          vectorSubtract3D(atom->x[i],atom->x[j],del);
+          array[ipair][n++] = radi+radj-vectorMag3D(del);
+          
+        }
+    }
+
+    if(cpflag)
+    {
+      double cp[3];
+      if(atom->superquadric_flag)
+      {
+        int contact_point_offset = get_history_offset("contact_point_offset");
+        cp[0] = hist[contact_point_offset];
+        cp[1] = hist[contact_point_offset + 1];
+        cp[2] = hist[contact_point_offset + 2];
+      } else {
         radi = atom->radius[i];
         radj = atom->radius[j];
-        vectorSubtract3D(atom->x[i],atom->x[j],del);
-        array[ipair][n++] = radi+radj-vectorMag3D(del);
-        
+        for(int dim = 0; dim < 3; dim++)
+          cp[dim] = (atom->x[i][dim]*radj + atom->x[j][dim]*radi)/(radi+radj);
+      }
+
+      array[ipair][n++] = cp[0];
+      array[ipair][n++] = cp[1];
+      array[ipair][n++] = cp[2];
     }
 
     ipair++;
@@ -692,7 +722,7 @@ void ComputePairGranLocal::add_wall_1(int iFMG,int idTri,int iP,double *contact_
 
 /* ---------------------------------------------------------------------- */
 
-void ComputePairGranLocal::add_wall_2(int i,double fx,double fy,double fz,double tor1,double tor2,double tor3,double *hist,double rsq)
+void ComputePairGranLocal::add_wall_2(int i,double fx,double fy,double fz,double tor1,double tor2,double tor3,double *hist,double rsq, double *normal)
 {
     double contactArea;
 
@@ -740,40 +770,28 @@ void ComputePairGranLocal::add_wall_2(int i,double fx,double fy,double fz,double
     }
     if(fnflag)
     {
-        double contact_point[3], fc[3],fn[3], normal[3];
-
-        contact_point[0] = array[ipair][n++];
-        contact_point[1] = array[ipair][n++];
-        contact_point[2] = array[ipair][n++];
+        double fc[3],fn[3];
 
         vectorConstruct3D(fc,fx,fy,fz);
-        vectorSubtract3D(contact_pos ? contact_pos :  contact_point,atom->x[i],normal);
-        vectorNormalize3D(normal);
         double fnmag = vectorDot3D(fc,normal);
         vectorScalarMult3D(normal,fnmag,fn);
 
-        array[ipair][n-3] = fn[0];
-        array[ipair][n-2] = fn[1];
-        array[ipair][n-1] = fn[2];
+        array[ipair][n++] = fn[0];
+        array[ipair][n++] = fn[1];
+        array[ipair][n++] = fn[2];
     }
     if(ftflag)
     {
-        double contact_point[3], fc[3],fn[3],ft[3], normal[3];
-
-        contact_point[0] = array[ipair][n++];
-        contact_point[1] = array[ipair][n++];
-        contact_point[2] = array[ipair][n++];
+        double fc[3],fn[3],ft[3];
 
         vectorConstruct3D(fc,fx,fy,fz);
-        vectorSubtract3D(contact_pos ? contact_pos : contact_point,atom->x[i],normal);
-        vectorNormalize3D(normal);
         double fnmag = vectorDot3D(fc,normal);
         vectorScalarMult3D(normal,fnmag,fn);
         vectorSubtract3D(fc,fn,ft);
 
-        array[ipair][n-3] = ft[0];
-        array[ipair][n-2] = ft[1];
-        array[ipair][n-1] = ft[2];
+        array[ipair][n++] = ft[0];
+        array[ipair][n++] = ft[1];
+        array[ipair][n++] = ft[2];
     }
     if(torqueflag)
     {
@@ -783,40 +801,28 @@ void ComputePairGranLocal::add_wall_2(int i,double fx,double fy,double fz,double
     }
     if(torquenflag)
     {
-        double  contact_point[3],torc[3],torn[3],normal[3];
-
-        contact_point[0] = array[ipair][n++];
-        contact_point[1] = array[ipair][n++];
-        contact_point[2] = array[ipair][n++];
+        double torc[3],torn[3];
 
         vectorConstruct3D(torc,tor1,tor2,tor3);
-        vectorSubtract3D(contact_pos ? contact_pos : contact_point,atom->x[i],normal);
-        vectorNormalize3D(normal);
         const double tornmag = vectorDot3D(torc,normal);
         vectorScalarMult3D(normal,tornmag,torn);
 
-        array[ipair][n-3] = torn[0];
-        array[ipair][n-2] = torn[1];
-        array[ipair][n-1] = torn[2];
+        array[ipair][n++] = torn[0];
+        array[ipair][n++] = torn[1];
+        array[ipair][n++] = torn[2];
     }
     if(torquetflag)
     {
-        double  contact_point[3],torc[3],torn[3],tort[3],normal[3];
-
-        contact_point[0] = array[ipair][n++];
-        contact_point[1] = array[ipair][n++];
-        contact_point[2] = array[ipair][n++];
+        double torc[3],torn[3],tort[3];
 
         vectorConstruct3D(torc,tor1,tor2,tor3);
-        vectorSubtract3D(contact_pos ? contact_pos : contact_point,atom->x[i],normal);
-        vectorNormalize3D(normal);
         const double tornmag = vectorDot3D(torc,normal);
         vectorScalarMult3D(normal,tornmag,torn);
         vectorSubtract3D(torc,torn,tort);
 
-        array[ipair][n-3] = tort[0];
-        array[ipair][n-2] = tort[1];
-        array[ipair][n-1] = tort[2];
+        array[ipair][n++] = tort[0];
+        array[ipair][n++] = tort[1];
+        array[ipair][n++] = tort[2];
     }
     if(histflag)
     {
@@ -877,3 +883,16 @@ double ComputePairGranLocal::memory_usage()
   double bytes = nmax*nvalues * sizeof(double);
   return bytes;
 }
+
+int ComputePairGranLocal::get_history_offset(const char * const name)
+{
+    if (pairgran)
+        return static_cast<PairGranProxy*>(pairgran)->get_history_offset(name);
+    else if (fixwall)
+        return fixwall->get_history_offset(name);
+    else
+    {
+        error->all(FLERR, "Internal error");
+        return -1;
+    }
+}
