@@ -38,8 +38,8 @@
     Copyright 2015-     DCS Computing GmbH, Linz
 ------------------------------------------------------------------------- */
 
-#include "string.h"
-#include "stdlib.h"
+#include <string.h>
+#include <stdlib.h>
 #include "atom.h"
 #include "update.h"
 #include "respa.h"
@@ -49,7 +49,7 @@
 #include "modify.h"
 #include "group.h"
 #include "comm.h"
-#include "math.h"
+#include <math.h>
 #include "vector_liggghts.h"
 #include "fix_cfd_coupling_convection_species.h"
 #include "fix_property_atom.h"
