@@ -69,6 +69,15 @@ FixHeatGranCond::FixHeatGranCond(class LAMMPS *lmp, int narg, char **arg) :
   FixHeatGran(lmp, narg, arg),
   fix_conductivity_(0),
   conductivity_(0),
+  store_contact_data_(false),
+  fix_conduction_contact_area_(0),
+  fix_n_conduction_contacts_(0),
+  fix_wall_heattransfer_coeff_(0),
+  fix_wall_temperature_(0),
+  conduction_contact_area_(0),
+  n_conduction_contacts_(0),
+  wall_heattransfer_coeff_(0),
+  wall_temp_(0),
   area_calculation_mode_(CONDUCTION_CONTACT_AREA_OVERLAP),
   fixed_contact_area_(0.),
   area_correction_flag_(0),
@@ -108,6 +117,15 @@ FixHeatGranCond::FixHeatGranCond(class LAMMPS *lmp, int narg, char **arg) :
       else error->fix_error(FLERR,this,"expecting 'yes' or 'no' after 'area_correction'");
       iarg_ += 2;
       hasargs = true;
+    } else if(strcmp(arg[iarg_],"store_contact_data") == 0) {
+      if (iarg_+2 > narg) error->fix_error(FLERR,this,"not enough arguments for keyword 'store_contact_data'");
+      if(strcmp(arg[iarg_+1],"yes") == 0)
+        store_contact_data_ = true;
+      else if(strcmp(arg[iarg_+1],"no") == 0)
+        store_contact_data_ = false;
+      else error->fix_error(FLERR,this,"expecting 'yes' or 'no' after 'store_contact_data'");
+      iarg_ += 2;
+      hasargs = true;
     } else if(strcmp(style,"heat/gran/conduction") == 0)
         error->fix_error(FLERR,this,"unknown keyword");
   }
@@ -130,6 +148,74 @@ FixHeatGranCond::~FixHeatGranCond()
 void FixHeatGranCond::post_create()
 {
   FixHeatGran::post_create();
+
+  // register contact storage
+  fix_conduction_contact_area_ = static_cast<FixPropertyAtom*>(modify->find_fix_property("contactAreaConduction","property/atom","scalar",0,0,this->style,false));
+  if(!fix_conduction_contact_area_ && store_contact_data_)
+  {
+    const char* fixarg[10];
+    fixarg[0]="contactAreaConduction";
+    fixarg[1]="all";
+    fixarg[2]="property/atom";
+    fixarg[3]="contactAreaConduction";
+    fixarg[4]="scalar";
+    fixarg[5]="no";
+    fixarg[6]="yes";
+    fixarg[7]="no";
+    fixarg[8]="0.";
+    fix_conduction_contact_area_ = modify->add_fix_property_atom(9,const_cast<char**>(fixarg),style);
+  }
+
+  fix_n_conduction_contacts_ = static_cast<FixPropertyAtom*>(modify->find_fix_property("nContactsConduction","property/atom","scalar",0,0,this->style,false));
+  if(!fix_n_conduction_contacts_ && store_contact_data_)
+  {
+    const char* fixarg[10];
+    fixarg[0]="nContactsConduction";
+    fixarg[1]="all";
+    fixarg[2]="property/atom";
+    fixarg[3]="nContactsConduction";
+    fixarg[4]="scalar";
+    fixarg[5]="no";
+    fixarg[6]="yes";
+    fixarg[7]="no";
+    fixarg[8]="0.";
+    fix_n_conduction_contacts_ = modify->add_fix_property_atom(9,const_cast<char**>(fixarg),style);
+  }
+
+  fix_wall_heattransfer_coeff_ = static_cast<FixPropertyAtom*>(modify->find_fix_property("wallHeattransferCoeff","property/atom","scalar",0,0,this->style,false));
+  if(!fix_wall_heattransfer_coeff_ && store_contact_data_)
+  {
+    const char* fixarg[10];
+    fixarg[0]="wallHeattransferCoeff";
+    fixarg[1]="all";
+    fixarg[2]="property/atom";
+    fixarg[3]="wallHeattransferCoeff";
+    fixarg[4]="scalar";
+    fixarg[5]="no";
+    fixarg[6]="yes";
+    fixarg[7]="no";
+    fixarg[8]="0.";
+    fix_wall_heattransfer_coeff_ = modify->add_fix_property_atom(9,const_cast<char**>(fixarg),style);
+  }
+
+  fix_wall_temperature_ = static_cast<FixPropertyAtom*>(modify->find_fix_property("wallTemp","property/atom","scalar",0,0,this->style,false));
+  if(!fix_wall_temperature_ && store_contact_data_)
+  {
+    const char* fixarg[10];
+    fixarg[0]="wallTemp";
+    fixarg[1]="all";
+    fixarg[2]="property/atom";
+    fixarg[3]="wallTemp";
+    fixarg[4]="scalar";
+    fixarg[5]="no";
+    fixarg[6]="yes";
+    fixarg[7]="no";
+    fixarg[8]="0.";
+    fix_wall_temperature_ = modify->add_fix_property_atom(9,const_cast<char**>(fixarg),style);
+  }
+
+  if(store_contact_data_ && (!fix_conduction_contact_area_ || !fix_n_conduction_contacts_ || !fix_wall_heattransfer_coeff_ || !fix_wall_temperature_))
+    error->one(FLERR,"internal error");
 }
 
 /* ---------------------------------------------------------------------- */
@@ -147,12 +233,28 @@ void FixHeatGranCond::pre_delete(bool unfixflag)
 int FixHeatGranCond::setmask()
 {
   int mask = FixHeatGran::setmask();
+  mask |= PRE_FORCE;
   mask |= POST_FORCE;
   return mask;
 }
 
 /* ---------------------------------------------------------------------- */
 
+void FixHeatGranCond::updatePtrs()
+{
+  FixHeatGran::updatePtrs();
+
+  if(store_contact_data_)
+  {
+    conduction_contact_area_ = fix_conduction_contact_area_->vector_atom;
+    n_conduction_contacts_ = fix_n_conduction_contacts_->vector_atom;
+    wall_heattransfer_coeff_ = fix_wall_heattransfer_coeff_->vector_atom;
+    wall_temp_ = fix_wall_temperature_->vector_atom;
+  }
+}
+
+/* ---------------------------------------------------------------------- */
+
 void FixHeatGranCond::init()
 {
   // initialize base class
@@ -217,6 +319,18 @@ void FixHeatGranCond::init()
 
 /* ---------------------------------------------------------------------- */
 
+void FixHeatGranCond::pre_force(int vflag)
+{
+    
+    if(store_contact_data_)
+    {
+        fix_wall_heattransfer_coeff_->set_all(0.);
+        fix_wall_temperature_->set_all(0.);
+    }
+}
+
+/* ---------------------------------------------------------------------- */
+
 void FixHeatGranCond::post_force(int vflag)
 {
 
@@ -291,6 +405,12 @@ void FixHeatGranCond::post_force_eval(int vflag,int cpl_flag)
 
   updatePtrs();
 
+  if(store_contact_data_)
+  {
+    fix_conduction_contact_area_->set_all(0.);
+    fix_n_conduction_contacts_->set_all(0.);
+  }
+
   // loop over neighbors of my atoms
   for (ii = 0; ii < inum; ii++) {
     i = ilist[ii];
@@ -343,8 +463,15 @@ void FixHeatGranCond::post_force_eval(int vflag,int cpl_flag)
               r = radsum - delta_n;
             }
 
-            //contact area of the two spheres
-            contactArea = - M_PI/4 * ( (r-radi-radj)*(r+radi-radj)*(r-radi+radj)*(r+radi+radj) )/(r*r);
+            if (r < fmax(radi, radj)) // one sphere is inside the other
+            {
+                // set contact area to area of smaller sphere
+                contactArea = fmin(radi,radj);
+                contactArea *= contactArea * M_PI;
+            }
+            else
+                //contact area of the two spheres
+                contactArea = - M_PI/4.0 * ( (r-radi-radj)*(r+radi-radj)*(r-radi+radj)*(r+radi+radj) )/(r*r);
         }
         else if (CONTACTAREA == CONDUCTION_CONTACT_AREA_CONSTANT)
             contactArea = fixed_contact_area_;
@@ -371,14 +498,25 @@ void FixHeatGranCond::post_force_eval(int vflag,int cpl_flag)
           directionalHeatFlux[i][0] += 0.50 * dirFlux[0];
           directionalHeatFlux[i][1] += 0.50 * dirFlux[1];
           directionalHeatFlux[i][2] += 0.50 * dirFlux[2];
+
+          if(store_contact_data_)
+          {
+              conduction_contact_area_[i] += contactArea;
+              n_conduction_contacts_[i] += 1.;
+          }
           if (newton_pair || j < nlocal)
           {
             heatFlux[j] -= flux;
             directionalHeatFlux[j][0] += 0.50 * dirFlux[0];
             directionalHeatFlux[j][1] += 0.50 * dirFlux[1];
             directionalHeatFlux[j][2] += 0.50 * dirFlux[2];
-          }
 
+            if(store_contact_data_)
+            {
+                conduction_contact_area_[j] += contactArea;
+                n_conduction_contacts_[j] += 1.;
+            }
+          }
         }
 
         if(cpl_flag && cpl) cpl->add_heat(i,j,flux);
@@ -386,8 +524,20 @@ void FixHeatGranCond::post_force_eval(int vflag,int cpl_flag)
     }
   }
 
-  if(newton_pair) fix_heatFlux->do_reverse_comm();
-  if(newton_pair) fix_directionalHeatFlux->do_reverse_comm();
+  if(newton_pair)
+  {
+    fix_heatFlux->do_reverse_comm();
+    fix_directionalHeatFlux->do_reverse_comm();
+    fix_conduction_contact_area_->do_reverse_comm();
+    fix_n_conduction_contacts_->do_reverse_comm();
+  }
+
+  if(!cpl_flag && store_contact_data_)
+  for(int i = 0; i < nlocal; i++)
+  {
+     if(n_conduction_contacts_[i] > 0.5)
+        conduction_contact_area_[i] /= n_conduction_contacts_[i];
+  }
 }
 
 /* ----------------------------------------------------------------------
