@@ -64,7 +64,6 @@
 
 #include "probability_distribution.h"
 #include "region_neighbor_list.h"
-#include "nonspherical_flags.h"
 
 using namespace LAMMPS_NS;
 using namespace FixConst;
@@ -82,8 +81,6 @@ FixInsert::FixInsert(LAMMPS *lmp, int narg, char **arg) :
 {
   if (narg < 7) error->fix_error(FLERR,this,"not enough arguments");
 
-  time_depend = 1;
-
   restart_global = 1;
 
   setup_flag = false;
@@ -155,10 +152,17 @@ FixInsert::FixInsert(LAMMPS *lmp, int narg, char **arg) :
       nflowrate = atof(arg[iarg+1]);
       iarg += 2;
       hasargs = true;
-    } else if (strcmp(arg[iarg],"insert_every") == 0 || strcmp(arg[iarg],"every") == 0) {
+    } else if (strcmp(arg[iarg],"insert_every_time") == 0 || strcmp(arg[iarg],"insert_every") == 0 || strcmp(arg[iarg],"every") == 0) {
       if (iarg+2 > narg) error->fix_error(FLERR,this,"");
       if(strcmp(arg[iarg+1],"once") == 0) insert_every = 0;
-      else insert_every = atoi(arg[iarg+1]);
+      else if(strcmp(arg[iarg],"insert_every_time") == 0)
+      {
+          if(!update->timestep_set)
+            error->fix_error(FLERR,this,"need so set 'timestep' before");
+          insert_every = static_cast<int>(atof(arg[iarg+1])/update->dt);
+      }
+      else
+          insert_every = atoi(arg[iarg+1]);
       if(insert_every < 0) error->fix_error(FLERR,this,"insert_every must be >= 0");
       iarg += 2;
       hasargs = true;
@@ -414,7 +418,7 @@ void FixInsert::init_defaults()
   vectorZeroize3D(v_insertFluct);
   vectorZeroize3D(omega_insert);
 
-  quatUnitize4D(quat_insert);
+  quatIdentity4D(quat_insert);
   quat_random_ = false;
 
   print_stats_during_flag = 1;
@@ -562,7 +566,10 @@ void FixInsert::init()
 
     // in case of new fix insert in a restarted simulation, have to add current time-step
     if(next_reneighbor > 0 && next_reneighbor < ntimestep)
+    {
+        
         error->fix_error(FLERR,this,"'start' step can not be before current step");
+    }
 
     if(property_name)
     {
@@ -572,6 +579,15 @@ void FixInsert::init()
 
 /* ---------------------------------------------------------------------- */
 
+void FixInsert::reset_timestep(bigint newstep,bigint oldstep)
+{
+    
+    next_reneighbor += (newstep-oldstep);
+    
+}
+
+/* ---------------------------------------------------------------------- */
+
 int FixInsert::min_type()
 {
     return type_min;
@@ -669,14 +685,14 @@ void FixInsert::pre_exchange()
   
   if(ninsert_this_local > ninsert_this_max_local)
   {
-      fix_distribution->random_init_list(ninsert_this_local);
+      init_list(ninsert_this_local);
       ninsert_this_max_local = ninsert_this_local;
   }
 
   // generate list of insertions
   // number of inserted particles can change if exact_number = 0
   
-  ninsert_this_local = fix_distribution->randomize_list(ninsert_this_local,groupbit,exact_number);
+  ninsert_this_local = generate_list(ninsert_this_local,groupbit,exact_number);
   
   MPI_Sum_Scalar(ninsert_this_local,ninsert_this,world);
 
@@ -689,6 +705,9 @@ void FixInsert::pre_exchange()
       // schedule next insertion
       if (insert_every && (!ninsert_exists || ninserted < ninsert))
         next_reneighbor += insert_every;
+      
+      else if(0 == insert_every)
+        next_reneighbor = -1;
 
       return;
   }
@@ -890,22 +909,6 @@ int FixInsert::distribute_ninsert_this(int ninsert_this)
     return ninsert_this_local;
 }
 
-/* ----------------------------------------------------------------------
-   count # of particles that could overlap
-   must loop local + ghost particles
-------------------------------------------------------------------------- */
-
-int FixInsert::count_nnear()
-{
-    int nall = atom->nlocal + atom->nghost;
-    int ncount = 0;
-
-    for(int i = 0; i < nall; i++)
-        ncount += is_nearby(i);
-
-    return ncount;
-}
-
 /* ----------------------------------------------------------------------
    fill neighbor list with nearby particles
 ------------------------------------------------------------------------- */
@@ -928,11 +931,12 @@ int FixInsert::load_xnear(int ninsert_this_local)
   {
     for (int i = 0; i < nall; ++i)
     {
-      if (is_nearby(i))
+      
+      if (is_nearby(i) && neighList.isInBoundingBox(x[i]) )
       {
 #ifdef SUPERQUADRIC_ACTIVE_FLAG
         if(atom->superquadric_flag and check_obb_flag)
-          neighList.insert_superquadric(x[i], radius[i], atom->quaternion[i], atom->shape[i]);
+          neighList.insert_superquadric(x[i], radius[i], atom->quaternion[i], atom->shape[i], atom->roundness[i]);
         else
           neighList.insert(x[i], radius[i]);
 #else
