@@ -144,23 +144,24 @@ FixTemplateMultisphere::FixTemplateMultisphere(LAMMPS *lmp, int narg, char **arg
         } else if(strcmp(arg[iarg],"fflag") == 0) {
             if (iarg+4 > narg)
                 error->fix_error(FLERR,this,"not enough arguments for 'fflag'");
-            if(strcmp(arg[iarg],"on"))
+            iarg++;
+            if(strcmp(arg[iarg],"on") == 0)
                 fflag_[0] = true;
-            else if(strcmp(arg[iarg],"off"))
+            else if(strcmp(arg[iarg],"off") == 0)
                 fflag_[0] = false;
             else
                 error->fix_error(FLERR,this,"expecting 'on or 'off' after 'fflag'");
             iarg++;
-            if(strcmp(arg[iarg],"on"))
+            if(strcmp(arg[iarg],"on") == 0)
                 fflag_[1] = true;
-            else if(strcmp(arg[iarg],"off"))
+            else if(strcmp(arg[iarg],"off") == 0)
                 fflag_[1] = false;
             else
                 error->fix_error(FLERR,this,"expecting 'on or 'off' after 'fflag'");
             iarg++;
-            if(strcmp(arg[iarg],"on"))
+            if(strcmp(arg[iarg],"on") == 0)
                 fflag_[2] = true;
-            else if(strcmp(arg[iarg],"off"))
+            else if(strcmp(arg[iarg],"off") == 0)
                 fflag_[2] = false;
             else
                 error->fix_error(FLERR,this,"expecting 'on or 'off' after 'fflag'");
@@ -169,23 +170,24 @@ FixTemplateMultisphere::FixTemplateMultisphere(LAMMPS *lmp, int narg, char **arg
         } else if(strcmp(arg[iarg],"tflag") == 0) {
             if (iarg+4 > narg)
                 error->fix_error(FLERR,this,"not enough arguments for 'fflag'");
-            if(strcmp(arg[iarg],"on"))
+            iarg++;
+            if(strcmp(arg[iarg],"on") == 0)
                 tflag_[0] = true;
-            else if(strcmp(arg[iarg],"off"))
+            else if(strcmp(arg[iarg],"off") == 0)
                 tflag_[0] = false;
             else
                 error->fix_error(FLERR,this,"expecting 'on or 'off' after 'fflag'");
             iarg++;
-            if(strcmp(arg[iarg],"on"))
+            if(strcmp(arg[iarg],"on") == 0)
                 tflag_[1] = true;
-            else if(strcmp(arg[iarg],"off"))
+            else if(strcmp(arg[iarg],"off") == 0)
                 tflag_[1] = false;
             else
                 error->fix_error(FLERR,this,"expecting 'on or 'off' after 'fflag'");
             iarg++;
-            if(strcmp(arg[iarg],"on"))
+            if(strcmp(arg[iarg],"on") == 0)
                 tflag_[2] = true;
-            else if(strcmp(arg[iarg],"off"))
+            else if(strcmp(arg[iarg],"off") == 0)
                 tflag_[2] = false;
             else
                 error->fix_error(FLERR,this,"expecting 'on or 'off' after 'fflag'");
@@ -212,7 +214,7 @@ FixTemplateMultisphere::FixTemplateMultisphere(LAMMPS *lmp, int narg, char **arg
     }
 
     // check if type has been defined
-    if(type_ < 1)
+    if(!lmp->wb && type_ < 1)
         error->fix_error(FLERR,this,"have to provide a type >=1");
 
     if((mass_set_ && !moi_set_) || (!mass_set_ && moi_set_))
@@ -229,7 +231,7 @@ void FixTemplateMultisphere::post_create()
   // bounding sphere, center of mass calculations in mother class
   // calculate inertia_ tensor and its eigensystem
 
-  // use density and volume from MC; mass calculated
+  // use density from input script and volume from MC; mass calculated
   if(!mass_set_)
   {
       // bounding sphere and center of mass
@@ -259,7 +261,7 @@ void FixTemplateMultisphere::post_create()
       calc_eigensystem();
       calc_displace_xcm_x_body();
   }
-  // use density specified and mass specified; volume calculated
+  // use density specified in input script  and mass specified in input script ; volume calculated
   else
   {
       calc_bounding_sphere();
@@ -275,6 +277,8 @@ void FixTemplateMultisphere::post_create()
       
   }
 
+  calc_volumeweight();
+
   print_info();
 }
 
@@ -304,13 +308,13 @@ void FixTemplateMultisphere::init()
         for(int j = i+1; j < n_fixes; j++)
         {
             ftms_j = static_cast<FixTemplateMultisphere*>(modify->find_fix_style_strict(style,j));
-            if(ftms_j != ftms_i && type_i == ftms_j->type())
+            if(!lmp->wb && ftms_j != ftms_i && type_i == ftms_j->type())
                 error->fix_error(FLERR,this,"multisphere template types have to be unique");
         }
     }
 
     // types are consecutive if no double usage and min/max is met
-    if(type_min != 1 || type_max != n_fixes)
+    if(!lmp->wb &&  (type_min != 1 || type_max != n_fixes))
         error->fix_error(FLERR,this,"multisphere template types have to be consecutive starting from 1");
 
     FixMultisphere *fix_multisphere = static_cast<FixMultisphere*>(modify->find_fix_style("multisphere", 0));
@@ -327,49 +331,47 @@ void FixTemplateMultisphere::calc_volumeweight()
 {
     double x_try[3],distSqr,n_hits;
 
-    int *hits_j = new int[nspheres];
-    int *hits_only_j = new int[nspheres];
+    bool *hits_j = new bool[nspheres];
 
-    vectorZeroizeN(hits_j,nspheres);
-    vectorZeroizeN(hits_only_j,nspheres);
+    vectorZeroizeN(volumeweight_,nspheres);
 
     // volumeweight_ = 0.5 + 0.5 * hits only in j / hits in j
 
+    int hits_tot = 0;
     for(int i = 0; i < ntry; i++)
     {
         generate_xtry(x_try);
         n_hits = 0;
+        vectorInitializeN(hits_j,nspheres,false);
 
         for(int j = 0; j < nspheres; j++)
         {
             distSqr = dist_sqr(j,x_try);
             if(distSqr < r_sphere[j]*r_sphere[j])
             {
-                hits_j[j]++;
+                hits_j[j] = true;
                 n_hits++;
             }
         }
 
-        if(n_hits == 1)
+        for(int j = 0; j < nspheres; j++)
         {
-            for(int j = 0; j < nspheres; j++)
-            {
-                distSqr = dist_sqr(j,x_try);
-                if(distSqr < r_sphere[j]*r_sphere[j])
-                {
-                    hits_only_j[j]++;
-                    n_hits++;
-                }
-            }
+            if(hits_j[j])
+                volumeweight_[j] += 1./static_cast<double>(n_hits);
         }
+
+        if (n_hits > 0)
+            hits_tot++;
     }
 
     // calculate volume weight
-    for(int j = 0; j < nspheres; j++)
-        volumeweight_[j] = 0.5 + 0.5 * hits_only_j[j]/hits_j[j];
+    if (hits_tot > 0) {
+        const double hits_tot_inv = 1./static_cast<double>(hits_tot);
+        for(int j = 0; j < nspheres; j++)
+            volumeweight_[j] *= hits_tot_inv;
+    }
 
     delete []hits_j;
-    delete []hits_only_j;
 }
 
 /* ----------------------------------------------------------------------
@@ -550,6 +552,7 @@ void FixTemplateMultisphere::randomize_single()
   for(int j = 0; j < nspheres; j++)
   {
       pti_m->radius_ins[j] = r_sphere[j];
+      pti_m->volumeweight[j] = volumeweight_[j];
       vectorCopy3D(x_sphere[j],pti_m->x_ins[j]);
       vectorCopy3D(displace_[j],pti_m->displace[j]);
   }
@@ -563,7 +566,7 @@ void FixTemplateMultisphere::randomize_single()
   vectorCopy3D(xcm_to_xb_body_,pti_m->xcm_to_xbound);
 
   vectorZeroize3D(pti_m->xcm_ins);
-  quatUnitize4D(pti_m->quat_ins);
+  quatIdentity4D(pti_m->quat_ins);
   vectorZeroize3D(pti_m->v_ins);
   vectorZeroize3D(pti_m->omega_ins);
 
@@ -574,7 +577,7 @@ void FixTemplateMultisphere::randomize_single()
 
 /* ----------------------------------------------------------------------*/
 
-void FixTemplateMultisphere::init_ptilist(int n_random_max)
+void FixTemplateMultisphere::init_ptilist(int n_random_max, const bool enforce_single, FixPropertyAtom * const fix_release)
 {
     n_pti_max = n_random_max;
     pti_list = (ParticleToInsert**) memory->smalloc(n_pti_max*sizeof(ParticleToInsert*),"pti_list");
@@ -621,6 +624,7 @@ void FixTemplateMultisphere::randomize_ptilist(int n_random,int distribution_gro
           for(int j = 0; j < nspheres; j++)
           {
               pti_m->radius_ins[j] = r_sphere[j];
+              pti_m->volumeweight[j] = volumeweight_[j];
               vectorCopy3D(x_sphere[j],pti_m->x_ins[j]);
               vectorCopy3D(displace_[j],pti_m->displace[j]);
           }
@@ -634,7 +638,7 @@ void FixTemplateMultisphere::randomize_ptilist(int n_random,int distribution_gro
           vectorCopy3D(xcm_to_xb_body_,pti_m->xcm_to_xbound);
 
           vectorZeroize3D(pti_m->xcm_ins);
-          quatUnitize4D(pti_m->quat_ins);
+          quatIdentity4D(pti_m->quat_ins);
           vectorZeroize3D(pti_m->v_ins);
           vectorZeroize3D(pti_m->omega_ins);
 
