@@ -43,6 +43,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <climits>
 #include "fix_template_sphere.h"
 #include "atom.h"
 #include "atom_vec.h"
@@ -109,171 +110,97 @@ FixTemplateSphere::FixTemplateSphere(LAMMPS *lmp, int narg, char **arg) :
     hasargs = false;
     if (strcmp(arg[iarg],"atom_type") == 0)
     {
-      if (iarg+2 > narg)
-        error->fix_error(FLERR,this,"not enough arguments");
-      atom_type=atoi(arg[iarg+1]);
-      if (atom_type < 1)
-        error->fix_error(FLERR,this,"invalid atom type (must be >=1)");
-      hasargs = true;
-      iarg += 2;
+        if (iarg+2 > narg)
+            error->fix_error(FLERR,this,"not enough arguments");
+        atom_type=atoi(arg[iarg+1]);
+        if (atom_type < 1)
+            error->fix_error(FLERR,this,"invalid atom type (must be >=1)");
+        hasargs = true;
+        iarg += 2;
     }
     else if (strcmp(arg[iarg],"region") == 0)
     {
-      if (iarg+2 > narg)
-        error->fix_error(FLERR,this,"not enough arguments");
-      int ireg = domain->find_region(arg[iarg+1]);
-      if (ireg < 0) error->fix_error(FLERR,this,"illegal region");
-      reg = domain->regions[ireg];
-      hasargs = true;
-      iarg += 2;
+        if (iarg+2 > narg)
+              error->fix_error(FLERR,this,"not enough arguments");
+        int ireg = domain->find_region(arg[iarg+1]);
+        if (ireg < 0)
+            error->fix_error(FLERR,this,"illegal region");
+        reg = domain->regions[ireg];
+        hasargs = true;
+        iarg += 2;
     }
     else if (strcmp(arg[iarg],"relative") == 0)
     {
-      if (iarg+2 > narg)
-        error->fix_error(FLERR,this,"not enough arguments for 'relative'");
-      if(0 == strcmp(arg[iarg+1],"yes"))
-        relative = true;
-      else if(0 == strcmp(arg[iarg+1],"no"))
-        relative = false;
-      else
-        error->fix_error(FLERR,this,"expecting 'yes' or 'no' after 'relative'");
-      hasargs = true;
-      iarg += 2;
+        if (iarg+2 > narg)
+            error->fix_error(FLERR,this,"not enough arguments for 'relative'");
+        if(0 == strcmp(arg[iarg+1],"yes"))
+            relative = true;
+        else if(0 == strcmp(arg[iarg+1],"no"))
+            relative = false;
+        else
+            error->fix_error(FLERR,this,"expecting 'yes' or 'no' after 'relative'");
+        hasargs = true;
+        iarg += 2;
     }
     else if (strcmp(arg[iarg],"region_variable") == 0)
     {
-      if (iarg+2 > narg)
-        error->fix_error(FLERR,this,"not enough arguments");
-      int ifix = modify->find_fix(arg[iarg+1]);
-      if (ifix < 0)
-        error->fix_error(FLERR,this,"illegal region/variable fix");
-      reg_var = static_cast<FixRegionVariable*>(modify->fix[ifix]);
-      hasargs = true;
-      iarg += 2;
+        if (iarg+2 > narg)
+            error->fix_error(FLERR,this,"not enough arguments");
+        int ifix = modify->find_fix(arg[iarg+1]);
+        if (ifix < 0)
+            error->fix_error(FLERR,this,"illegal region/variable fix");
+        reg_var = static_cast<FixRegionVariable*>(modify->fix[ifix]);
+        hasargs = true;
+        iarg += 2;
     }
     else if (strcmp(arg[iarg],"volume_limit") == 0)
     {
-      if (iarg+2 > narg)
-        error->fix_error(FLERR,this,"not enough arguments for 'volume_limit'");
-      vol_limit = atof(arg[iarg+1]);
-      if(vol_limit <= 0)
-        error->fix_error(FLERR,this,"volume_limit > 0 required");
-      hasargs = true;
-      iarg += 2;
+        if (iarg+2 > narg)
+            error->fix_error(FLERR,this,"not enough arguments for 'volume_limit'");
+        vol_limit = atof(arg[iarg+1]);
+        if(vol_limit <= 0)
+            error->fix_error(FLERR,this,"volume_limit > 0 required");
+        hasargs = true;
+        iarg += 2;
     }
-    else if (strcmp(arg[iarg],"radius") == 0) {
-      hasargs = true;
-      if(strcmp(this->style,"particletemplate/sphere"))
-        error->fix_error(FLERR,this,"keyword radius only valid for particletemplate/sphere");
-      if (iarg+3 > narg)
-        error->all(FLERR,"Illegal fix particletemplate/sphere command, not enough arguments");
-      pdf_radius = new PDF(error);
-      if (strcmp(arg[iarg+1],"constant") == 0)
-      {
-          double value = atof(arg[iarg+2])*force->cg();
-          if( value <= 0.)
-            error->all(FLERR,"Illegal fix particletemplate/sphere command, radius must be >= 0");
-          pdf_radius->set_params<RANDOM_CONSTANT>(value);
-          iarg += 3;
-      }
-      else if (strcmp(arg[iarg+1],"uniform") == 0)
-      {
-          if(iarg+5 > narg)
-              error->fix_error(FLERR,this,"not enough arguments for uniform");
-          if(strcmp(arg[iarg+2],"mass") == 0)
-              pdf_radius->activate_mass_shift();
-          else if(strcmp(arg[iarg+2],"number"))
-              error->fix_error(FLERR,this,"expecting 'mass' or 'number'");
-          if(force->cg() > 1.)
-              error->fix_error(FLERR,this,"cannot use distribution with coarse-graining.");
-          double min = atof(arg[iarg+3]);
-          double max = atof(arg[iarg+4]);
-          if( min <= 0. || max <= 0. || min >= max)
-            error->fix_error(FLERR,this,"illegal min or max value for radius");
-          pdf_radius->set_params<RANDOM_UNIFORM>(min,max);
-          iarg += 5;
-      }
-      else if (strcmp(arg[iarg+1],"gaussian") == 0)
-      {
-          if(iarg+5 > narg)
-              error->fix_error(FLERR,this,"not enough arguments for gaussian");
-          if(strcmp(arg[iarg+2],"mass") == 0)
-              pdf_radius->activate_mass_shift();
-          else if(strcmp(arg[iarg+2],"number"))
-              error->fix_error(FLERR,this,"expecting 'mass' or 'number'");
-          if(force->cg() > 1.)
-              error->fix_error(FLERR,this,"cannot use distribution with coarse-graining.");
-          double mu = atof(arg[iarg+3]);
-          double sigma = atof(arg[iarg+4]);
-          if( mu <= 0. ) error->fix_error(FLERR,this,"illegal mu value for radius");
-          if( sigma <= 0. ) error->fix_error(FLERR,this,"illegal sigma value for radius");
-          pdf_radius->set_params<RANDOM_GAUSSIAN>(mu,sigma);
-          iarg += 5;
-      }
-      else if (strcmp(arg[iarg+1],"lognormal") == 0)
-      {
-          if(iarg+5 > narg)
-              error->fix_error(FLERR,this,"not enough arguments for lognormal");
-          if(strcmp(arg[iarg+2],"mass") == 0)
-              pdf_radius->activate_mass_shift();
-          else if(strcmp(arg[iarg+2],"number"))
-              error->fix_error(FLERR,this,"expecting 'mass' or 'number'");
-          if(force->cg() > 1.)
-              error->fix_error(FLERR,this,"cannot use distribution with coarse-graining.");
-          double mu = atof(arg[iarg+3]);
-          double sigma = atof(arg[iarg+4]);
-          if( sigma <= 0. ) error->fix_error(FLERR,this,"illegal sigma value for radius");
-          pdf_radius->set_params<RANDOM_LOGNORMAL>(mu,sigma);
-          iarg += 5;
-      }
-      else error->fix_error(FLERR,this,"invalid radius random style");
+    else if (strcmp(arg[iarg],"radius") == 0)
+    {
+        hasargs = true;
+        if(strcmp(this->style,"particletemplate/sphere"))
+            error->fix_error(FLERR,this,"keyword radius only valid for particletemplate/sphere");
+        if (iarg+3 > narg)
+            error->all(FLERR,"Illegal fix particletemplate/sphere command, not enough arguments");
+        pdf_radius = new PDF(error);
+        if (strcmp(arg[iarg+1],"constant") == 0)
+        {
+            double value = atof(arg[iarg+2])*force->cg(atom_type);
+            if( value <= 0.)
+              error->all(FLERR,"Illegal fix particletemplate/sphere command, radius must be >= 0");
+            pdf_radius->set_params<RANDOM_CONSTANT>(value);
+            iarg += 3;
+        }
+        else
+            error->fix_error(FLERR,this,"invalid radius random style");
     }
-    else if (strcmp(arg[iarg],"density") == 0) {
-      hasargs = true;
-      if (iarg+3 > narg) error->fix_error(FLERR,this,"not enough arguments");
-      pdf_density = new PDF(error);
-      if (strcmp(arg[iarg+1],"constant") == 0)
-      {
-          double value = atof(arg[iarg+2]);
-          if( value <= 0.) error->fix_error(FLERR,this,"density must be >= 0");
-          pdf_density->set_params<RANDOM_CONSTANT>(value);
-          iarg += 3;
-      }
-      else if (strcmp(arg[iarg+1],"uniform") == 0)
-      {
-          if (iarg+4 > narg) error->fix_error(FLERR,this,"not enough arguments");
-          double min = atof(arg[iarg+2]);
-          double max = atof(arg[iarg+3]);
-          if( min <= 0. || max <= 0. || min >= max)
-            error->fix_error(FLERR,this,"illegal min or max value for density");
-          pdf_density->set_params<RANDOM_UNIFORM>(min,max);
-          iarg += 4;
-      }
-      else if (strcmp(arg[iarg+1],"gaussian") == 0)
-      {
-          if (iarg+4 > narg)
+    else if (strcmp(arg[iarg],"density") == 0)
+    {
+        hasargs = true;
+        if (iarg+3 > narg)
             error->fix_error(FLERR,this,"not enough arguments");
-          double mu = atof(arg[iarg+2]);
-          double sigma = atof(arg[iarg+3]);
-          if( mu <= 0. )
-            error->fix_error(FLERR,this,"illegal mu value for density");
-          if( sigma <= 0. ) error->all(FLERR,"illegal sigma value for density");
-          pdf_density->set_params<RANDOM_GAUSSIAN>(mu,sigma);
-          iarg += 4;
-      }
-      else if (strcmp(arg[iarg+1],"lognormal") == 0)
-      {
-          if (iarg+4 > narg)
-            error->fix_error(FLERR,this,"not enough arguments");
-          double mu = atof(arg[iarg+2]);
-          double sigma = atof(arg[iarg+3]);
-          if( sigma <= 0. ) error->all(FLERR,"illegal sigma value for density");
-          pdf_density->set_params<RANDOM_LOGNORMAL>(mu,sigma);
-          iarg += 4;
-      }
-      else error->fix_error(FLERR,this,"invalid density random style");
-      
-    } else if(strcmp(arg[iarg],"additional_group") == 0) {
+        pdf_density = new PDF(error);
+        if (strcmp(arg[iarg+1],"constant") == 0)
+        {
+            double value = atof(arg[iarg+2]);
+            if (value <= 0.)
+                error->fix_error(FLERR,this,"density must be >= 0");
+            pdf_density->set_params<RANDOM_CONSTANT>(value);
+            iarg += 3;
+        }
+        else
+            error->fix_error(FLERR,this,"invalid density random style");
+    }
+    else if(strcmp(arg[iarg],"additional_group") == 0)
+    {
         if (iarg+2 > narg)
             error->fix_error(FLERR,this,"not enough arguments for 'additional_group'");
         iarg++;
@@ -282,16 +209,20 @@ FixTemplateSphere::FixTemplateSphere(LAMMPS *lmp, int narg, char **arg) :
         groupbit = groupbit | group->bitmask[iAddGroup];
         
         hasargs = true;
-    } else if(strcmp(style,"particletemplate/sphere") == 0)
+    }
+    else if(strcmp(style,"particletemplate/sphere") == 0)
         error->fix_error(FLERR,this,"unrecognized keyword");
   }
 
-  if(pdf_density == NULL) error->fix_error(FLERR,this,"have to define 'density'");
+  if(pdf_density == NULL)
+      error->fix_error(FLERR,this,"have to define 'density'");
 
   // end here for derived classes
-  if(strcmp(this->style,"particletemplate/sphere"))return;
+  if(strcmp(this->style,"particletemplate/sphere"))
+      return;
 
-  if(pdf_radius == NULL) error->fix_error(FLERR,this,"have to define 'radius'");
+  if(pdf_radius == NULL)
+      error->fix_error(FLERR,this,"have to define 'radius'");
 
   // set mass and volume expectancy
   volume_expect = cubic_expectancy(pdf_radius)*4.*M_PI/3.;
@@ -358,13 +289,13 @@ void FixTemplateSphere::randomize_single()
 
 /* ----------------------------------------------------------------------*/
 
-void FixTemplateSphere::init_ptilist(int n_random_max)
+void FixTemplateSphere::init_ptilist(int n_random_max, const bool enforce_single, FixPropertyAtom * const fix_release)
 {
     if(pti_list) error->one(FLERR,"invalid FixTemplateSphere::init_list()");
     n_pti_max = n_random_max;
     pti_list = (ParticleToInsert**) memory->smalloc(n_pti_max*sizeof(ParticleToInsert*),"pti_list");
     for(int i = 0; i < n_pti_max; i++)
-       pti_list[i] = new ParticleToInsert(lmp);
+       pti_list[i] = new ParticleToInsert(lmp, 1, fix_release);
 }
 
 /* ----------------------------------------------------------------------*/
@@ -416,6 +347,48 @@ void FixTemplateSphere::randomize_ptilist(int n_random,int distribution_groupbit
 
 /* ----------------------------------------------------------------------*/
 
+void FixTemplateSphere::direct_set_ptlist(const int i, const void * const data, const int distribution_groupbit, const int distorder)
+{
+    const PARTICLE_PACKING::Sphere * const sphere = static_cast<const PARTICLE_PACKING::Sphere * const>(data);
+    pti_list[i]->atom_type = atom_type;
+    const double radius = sphere->get_radius();
+    pti_list[i]->radius_ins[0] = radius;
+    pti_list[i]->density_ins = sphere->get_density();
+    pti_list[i]->volume_ins = radius*radius*radius*4.1887902047863909;
+    pti_list[i]->mass_ins = pti_list[i]->density_ins*pti_list[i]->volume_ins;
+    pti_list[i]->id_ins = sphere->get_id();
+
+    // set fix_property_atom
+    if (pti_list[i]->fix_property || pti_list[i]->fix_property_value)
+        error->one(FLERR, "Ensure that set_property is not used in fix insert");
+    if (sphere->n_fix_properties() > 0)
+    {
+        const int n = sphere->n_fix_properties();
+        pti_list[i]->n_fix_property = n;
+        pti_list[i]->fix_property = new FixPropertyAtom*[n];
+        pti_list[i]->fix_property_value = new double*[n];
+        for (int j = 0; j < n; j++)
+        {
+            pti_list[i]->fix_property[j] = sphere->get_fix_property(j);
+            const int m = sphere->fix_property_nentries(j);
+            pti_list[i]->fix_property_value[j] = new double[m];
+            for (int k = 0; k < m; k++)
+                pti_list[i]->fix_property_value[j][k] = sphere->fix_property_value(j, k);
+        }
+    }
+
+    // init insertion position
+    vectorZeroize3D(pti_list[i]->x_ins[0]);
+    vectorZeroize3D(pti_list[i]->v_ins);
+    vectorZeroize3D(pti_list[i]->omega_ins);
+
+    pti_list[i]->groupbit = groupbit | distribution_groupbit; 
+
+    pti_list[i]->distorder = distorder;
+}
+
+/* ----------------------------------------------------------------------*/
+
 double FixTemplateSphere::min_rad()
 {
     
@@ -537,5 +510,51 @@ void FixTemplateSphere::restart(char *buf)
   }
 
   random_insertion->reset(seed_insertion);
-  random_mc->reset(seed_mc);
+}
+
+/* ----------------------------------------------------------------------
+   generate hash to identify this template
+------------------------------------------------------------------------- */
+
+unsigned int FixTemplateSphere::generate_hash()
+{
+    unsigned int hash = 0;
+    unsigned int start = seed_insertion*420001; // it's magic
+    add_hash_value(atom_type, start, hash);
+    add_hash_value(pdf_radius->rand_style(), start, hash);
+    add_hash_value(expectancy(pdf_radius), start, hash);
+    add_hash_value(cubic_expectancy(pdf_radius), start, hash);
+    add_hash_value(pdf_density->rand_style(), start, hash);
+    add_hash_value(expectancy(pdf_density), start, hash);
+    add_hash_value(cubic_expectancy(pdf_density), start, hash);
+    return hash;
+}
+
+void FixTemplateSphere::add_hash_value(const int value, unsigned int &start, unsigned int &hash)
+{
+    if (value >= 0)
+        hash = hash*start + (unsigned int)value;
+    else
+        hash = hash*start + (unsigned int)(-value) + INT_MAX;
+    start = start*seed_insertion;
+}
+
+void FixTemplateSphere::add_hash_value(double value, unsigned int &start, unsigned int &hash)
+{
+    int ivalue = 0;
+    if (value < 0)
+        value *= -1.0;
+    if (value > 1e-50)
+    {
+        while (value > 1e6)
+            value *= 1e-6;
+
+        while (value < 1e0)
+            value *= 1e6;
+        int high = (int) value;
+        double remainder = (value - (double)high)*1e6;
+        int low = (int) remainder;
+        ivalue = high + low;
+    }
+    add_hash_value(ivalue, start, hash);
 }
