@@ -87,12 +87,21 @@ namespace MODEL_PARAMS
   static const char * ROUGHNESS_RELATIVE = "roughnessRelative";
   static const char * ROLLING_STIFFNESS = "rollingStiffness";
   static const char * NORMAL_DAMPING_COEFFICIENT = "normalDampingCoefficient";
-  static const char * TANGENTIAL_DAMPING_COEFFICIENT = "tangentialDampingCoefficient";
   static const char * TANGENTIAL_STIFFNESS = "tangentialStiffness";
   static const char * INITIAL_COHESIVE_STRESS = "initialCohesiveStress";
   static const char * MAX_COHESIVE_STRESS = "maxCohesiveStress";
   static const char * COHESION_STRENGTH = "cohesionStrength";
-
+  static const char * PULL_OFF_FORCE = "pullOffForce";
+  static const char * OVERLAP_EXPONENT = "overlapExponent";
+  static const char * ADHESION_EXPONENT = "adhesionExponent";
+  static const char * SURFACE_ENERGY = "surfaceEnergy";
+  static const char * TANGENTIAL_MULTIPLIER = "tangentialMultiplier";
+  static const char * YIELD_RATIO = "coefficientYieldRatio";
+  static const char * FRICTION_VISCOSITY = "FrictionViscosity";
+  static const char * COEFFICIENT_FRICTION_STIFFNESS = "coeffFrictionStiffness";
+  static const char * COEFFICIENT_ROLLING_FRICTION_STIFFNESS = "coeffRollingStiffness";
+  static const char * UNLOADING_STIFFNESS = "UnloadingStiffness";
+  static const char * LOADING_STIFFNESS = "LoadingStiffness";
   /* -----------------------------------------------------------------------
    * Utility functions
    * ----------------------------------------------------------------------- */
@@ -352,9 +361,16 @@ namespace MODEL_PARAMS
       if(sanity_checks && (0 == lmp->modify->n_fixes_style("bubble")) && (!registry.getLAMMPS()->atom->get_properties()->allow_soft_particles()))
       {
         if(strcmp(lmp->update->unit_style,"si") == 0  && Yi < 5e6)
-          lmp->error->all(FLERR,"youngsModulus >= 5e6 required for SI units");
+          lmp->error->all(FLERR,"youngsModulus >= 5e6 required for SI units (use command 'soft_particles yes' to override)");
         if(strcmp(lmp->update->unit_style,"cgs") == 0 && Yi < 5e5)
-          lmp->error->all(FLERR,"youngsModulus >= 5e5 required for CGS units");
+          lmp->error->all(FLERR,"youngsModulus >= 5e5 required for CGS units (use command 'soft_particles yes' to override)");
+      }
+      if(sanity_checks && !registry.getLAMMPS()->atom->get_properties()->allow_hard_particles())
+      {
+        if(strcmp(lmp->update->unit_style,"si") == 0  && Yi > 1e9)
+          lmp->error->all(FLERR,"youngsModulus <= 1e9 required for SI units (use command 'hard_particles yes' to override)");
+        if(strcmp(lmp->update->unit_style,"cgs") == 0 && Yi > 1e8)
+          lmp->error->all(FLERR,"youngsModulus <= 1e8 required for CGS units (use command 'hard_particles yes' to override)");
       }
 
       vec->data[i] = Yi;
@@ -723,13 +739,16 @@ namespace MODEL_PARAMS
 
     MatrixProperty * matrix = new MatrixProperty(max_type+1, max_type+1);
     FixPropertyGlobal * k_n1 = registry.getGlobalProperty("kn","property/global","peratomtypepair",max_type,max_type,caller);
-    const double cg = lmp->force->cg();
 
     for(int i=1;i< max_type+1; i++)
     {
+     const double cg_i = lmp->force->cg(i);
      for(int j=1;j<max_type+1;j++)
      {
-       const double k_n = cg*k_n1->compute_array(i-1,j-1);
+       const double cg_j = lmp->force->cg(j);
+       if(cg_i != cg_j)
+        lmp->error->all(FLERR,"per-type coarse-graining factors must be equal when property 'kn' is used");
+       const double k_n = cg_i*k_n1->compute_array(i-1,j-1);
        matrix->data[i][j] = k_n;
      }
     }
@@ -753,13 +772,16 @@ namespace MODEL_PARAMS
 
     MatrixProperty * matrix = new MatrixProperty(max_type+1, max_type+1);
     FixPropertyGlobal * gamma_n1 = registry.getGlobalProperty("gamman","property/global","peratomtypepair",max_type,max_type,caller);
-    const double cg = lmp->force->cg();
 
     for(int i=1;i< max_type+1; i++)
     {
+      const double cg_i = lmp->force->cg(i);
       for(int j=1;j<max_type+1;j++)
       {
-        const double k_n = (1./cg)*gamma_n1->compute_array(i-1,j-1);
+        const double cg_j = lmp->force->cg(j);
+        if(cg_i != cg_j)
+         lmp->error->all(FLERR,"per-type coarse-graining factors must be equal when property 'gamman' is used");
+        const double k_n = (1./cg_i)*gamma_n1->compute_array(i-1,j-1);
         matrix->data[i][j] = k_n;
       }
     }
@@ -783,14 +805,17 @@ namespace MODEL_PARAMS
 
     MatrixProperty * matrix = new MatrixProperty(max_type+1, max_type+1);
     FixPropertyGlobal * gamma_n1 = registry.getGlobalProperty("gamman_abs","property/global","peratomtypepair",max_type,max_type,caller);
-    const double cg = lmp->force->cg();
 
     for(int i=1;i< max_type+1; i++)
     {
+      const double cg_i = lmp->force->cg(i);
       for(int j=1;j<max_type+1;j++)
       {
-        const double k_n = cg*cg*gamma_n1->compute_array(i-1,j-1);
-        matrix->data[i][j] = k_n;
+        const double cg_j = lmp->force->cg(j);
+        if(cg_i != cg_j)
+         lmp->error->all(FLERR,"per-type coarse-graining factors must be equal when property 'gamman_abs' is used");
+        const double gamma = cg_i*cg_i*gamma_n1->compute_array(i-1,j-1);
+        matrix->data[i][j] = gamma;
       }
     }
 
@@ -833,6 +858,10 @@ namespace MODEL_PARAMS
   }
 
   /* ---------------------------------------------------------------------- */
+  MatrixProperty* createPullOffForce(PropertyRegistry & registry, const char * caller, bool)
+  {
+    return createPerTypePairProperty(registry, PULL_OFF_FORCE, caller);
+  }
 
   ScalarProperty* createRoughnessRelative(PropertyRegistry & registry, const char * caller, bool)
   {
@@ -864,16 +893,198 @@ namespace MODEL_PARAMS
     return createPerTypePairProperty(registry, INITIAL_COHESIVE_STRESS, caller);
   }
   /* ---------------------------------------------------------------------- */
-
-  MatrixProperty* createMaxCohesiveStress(PropertyRegistry & registry, const char * caller, bool)
+   ScalarProperty* createOverlapExponent(PropertyRegistry & registry, const char * caller, bool)
   {
-    return createPerTypePairProperty(registry, MAX_COHESIVE_STRESS, caller);
+    return createScalarProperty(registry, OVERLAP_EXPONENT, caller);
   }
+
+  ScalarProperty* createAdhesionExponent(PropertyRegistry & registry, const char * caller, bool)
+  {
+    return createScalarProperty(registry, ADHESION_EXPONENT, caller);
+  }
+
   /* ---------------------------------------------------------------------- */
 
-  MatrixProperty* createCohesionStrength(PropertyRegistry & registry, const char * caller, bool)
+  MatrixProperty* createSurfaceEnergy(PropertyRegistry & registry, const char * caller, bool)
   {
-    return createPerTypePairProperty(registry, COHESION_STRENGTH, caller);
+    return createPerTypePairProperty(registry, SURFACE_ENERGY, caller);
   }
+
   /* ---------------------------------------------------------------------- */
+
+  MatrixProperty* createTangentialMultiplier(PropertyRegistry & registry, const char * caller, bool)
+  {
+    return createPerTypePairProperty(registry, TANGENTIAL_MULTIPLIER, caller);
+  }
+
+  /* ---------------------------------------------------------------------- */
+
+  // Thornton & Ning
+
+  VectorProperty * createYieldRatio(PropertyRegistry & registry, const char * caller, bool sanity_checks)
+  {
+    LAMMPS * lmp = registry.getLAMMPS();
+    const int max_type = registry.max_type();
+
+    VectorProperty * vec = new VectorProperty(max_type+1);
+    FixPropertyGlobal * v = registry.getGlobalProperty(YIELD_RATIO,"property/global","peratomtype",max_type,0,caller);
+
+    for(int i=1; i < max_type+1; i++)
+    {
+      const double vi = v->compute_vector(i-1);
+
+      // error checks on v
+      if(sanity_checks)
+      {
+        if(vi < 0. || vi > 1)
+          lmp->error->all(FLERR,"0 <= poissonsRatio <= 1 required");
+      }
+
+      vec->data[i] = vi;
+    }
+
+    return vec;
+  }
+
+  /* ----------------------Luding's tangential parameter-frictional viscous dampening------------------------------------------------ */
+
+    MatrixProperty * createCoeffFricVisc(PropertyRegistry & registry, const char * caller, bool sanity_checks)
+      {
+        LAMMPS * lmp = registry.getLAMMPS();
+        const int max_type = registry.max_type();
+
+        MatrixProperty * matrix = new MatrixProperty(max_type+1, max_type+1);
+        FixPropertyGlobal * coeffFricVisc = registry.getGlobalProperty(FRICTION_VISCOSITY,"property/global","peratomtypepair",max_type,max_type,caller);
+
+        for(int i=1;i< max_type+1; i++)
+        {
+         for(int j=1;j<max_type+1;j++)
+         {
+           const double mu = coeffFricVisc->compute_array(i-1,j-1);
+
+           if(sanity_checks)
+           {
+             if(mu <= 0.)
+              lmp->error->all(FLERR,"coeffFricVisc > 0 required");
+           }
+
+           matrix->data[i][j] = mu;
+         }
+        }
+
+        return matrix;
+      }
+
+    /* ---------------------------------------------------------------------- */
+
+    MatrixProperty* createCoeffFrictionStiffness(PropertyRegistry & registry, const char * caller, bool)
+      {
+        return createPerTypePairProperty(registry, COEFFICIENT_FRICTION_STIFFNESS, caller);
+      }
+
+    MatrixProperty* createCoeffRollingStiffness(PropertyRegistry & registry, const char * caller, bool)
+      {
+        return createPerTypePairProperty(registry, COEFFICIENT_ROLLING_FRICTION_STIFFNESS, caller);
+      }
+
+    /* --------------------- UnLoading Slope for Luding and Edinburgh Linear -----------------*/
+    MatrixProperty* createUnloadingStiffness(PropertyRegistry & registry, const char * caller, bool)
+      {
+        return createPerTypePairProperty(registry, UNLOADING_STIFFNESS, caller);
+      }
+
+    /* -------------------------- Loading Slope for Luding and Edinburgh Linear -------------------- */
+    MatrixProperty* createLoadingStiffness(PropertyRegistry & registry, const char * caller, bool)
+    {
+      return createPerTypePairProperty(registry, LOADING_STIFFNESS, caller);
+    }
+
+    MatrixProperty* createMaxCohesiveStress(PropertyRegistry & registry, const char * caller, bool)
+    {
+      return createPerTypePairProperty(registry, MAX_COHESIVE_STRESS, caller);
+    }
+
+    /* ---------------------------------------------------------------------- */
+
+    MatrixProperty* createCohesionStrength(PropertyRegistry & registry, const char * caller, bool)
+    {
+      return createPerTypePairProperty(registry, COHESION_STRENGTH, caller);
+    }
+
+    /* ---------------------------------------------------------------------- */
+
+    // Dissipation matrix creation (fiber & bond model) (1/timeScale)
+
+    MatrixProperty* createDissipationMatrix(PropertyRegistry & registry, const char * name, const char * caller, bool sanity_checks)
+    {
+        LAMMPS * lmp = registry.getLAMMPS();
+        const int max_type = registry.max_type();
+
+        MatrixProperty * matrix = new MatrixProperty(max_type+1, max_type+1);
+        FixPropertyGlobal * property = registry.getGlobalProperty(name,"property/global","peratomtypepair",max_type,max_type,caller);
+
+        for(int i=1;i< max_type+1; i++)
+        {
+            for(int j=1;j<max_type+1;j++)
+            {
+                const double gamma_one = property->compute_array(i-1,j-1);
+
+                if(sanity_checks)
+                {
+                    if(gamma_one < lmp->update->dt)
+                    {
+                        char buffer [100];
+                        sprintf(buffer,"%s > time-step size required",name);
+                        lmp->error->all(FLERR,buffer);
+                    }
+                }
+
+                matrix->data[i][j] = 1./gamma_one; //save already inverted parameter
+            }
+        }
+
+        return matrix;
+    }
+
+    /* ---------------------------------------------------------------------- */
+
+    // Attrition properties
+    MatrixProperty* createAbrasiveWearSeverity(PropertyRegistry & registry, const char * caller, bool sanity_checks)
+    {
+        MatrixProperty* wearSeverityMatrix = MODEL_PARAMS::createPerTypePairProperty(registry, "abrasiveWearSeverity", caller);
+        return wearSeverityMatrix;
+    }
+
+    MatrixProperty* createErosiveWearSeverity(PropertyRegistry & registry, const char * caller, bool sanity_checks)
+    {
+        MatrixProperty* wearSeverityMatrix = MODEL_PARAMS::createPerTypePairProperty(registry, "erosiveWearSeverity", caller);
+        return wearSeverityMatrix;
+    }
+
+    MatrixProperty* createVelocityLimit(PropertyRegistry & registry, const char * caller, bool sanity_checks)
+    {
+        MatrixProperty* velocityLimitMatrix = MODEL_PARAMS::createPerTypePairProperty(registry, "velocityLimit", caller);
+        return velocityLimitMatrix;
+    }
+
+    MatrixProperty* createForceLimit(PropertyRegistry & registry, const char * caller, bool sanity_checks)
+    {
+        MatrixProperty* forceLimitMatrix = MODEL_PARAMS::createPerTypePairProperty(registry, "forceLimit", caller);
+        return forceLimitMatrix;
+    }
+
+    VectorProperty* createAttritionLimit(PropertyRegistry & registry, const char * caller, bool sanity_checks)
+    {
+        VectorProperty* attritionLimit = MODEL_PARAMS::createPerTypeProperty(registry, "attritionLimit", caller);
+        LAMMPS *lmp = registry.getLAMMPS();
+        for (int i = 1; i < registry.max_type()+1; i++)
+        {
+            const double limit = attritionLimit->data[i];
+            if (limit < 0 || limit > 1)
+                lmp->error->all(FLERR, "Attrition limit needs to be between 0 and 1");
+        }
+        return attritionLimit;
+    }
+
+    /* ---------------------------------------------------------------------- */
 }
