@@ -59,7 +59,7 @@ namespace Walls {
   public:
     typedef FixWallGran ParentType;
     virtual ~IGranularWall();
-    virtual void settings(int nargs, char ** args) = 0;
+    virtual void settings(int nargs, char ** args, IContactHistorySetup *hsetup) = 0;
     virtual void init_granular() = 0;
 
     virtual bool checkSurfaceIntersect(ContactModels::SurfacesIntersectData & sidata) = 0;
