@@ -62,6 +62,7 @@
 #include "dump.h"
 #include "memory.h"
 #include "error.h"
+#include "fix_multisphere.h"
 
 using namespace LAMMPS_NS;
 
@@ -76,7 +77,8 @@ enum{LT,LE,GT,GE,EQ,NEQ,BETWEEN};
    initialize group memory
 ------------------------------------------------------------------------- */
 
-Group::Group(LAMMPS *lmp) : Pointers(lmp)
+Group::Group(LAMMPS *lmp) : Pointers(lmp),
+  fix_ms_(NULL)
 {
   MPI_Comm_rank(world,&me);
 
@@ -461,6 +463,15 @@ void Group::create(const char *name, int *flag)
   }
 }
 
+/* ----------------------------------------------------------------------
+   general setup function before run
+------------------------------------------------------------------------- */
+
+void Group::init()
+{
+  fix_ms_ =  static_cast<FixMultisphere*>(modify->find_fix_style("multisphere",0));
+}
+
 /* ----------------------------------------------------------------------
    add flagged atoms to a new or existing group
 ------------------------------------------------------------------------- */
@@ -625,6 +636,67 @@ bigint Group::count(int igroup, int iregion)
   return nall;
 }
 
+/* ----------------------------------------------------------------------
+   count atoms in group
+------------------------------------------------------------------------- */
+
+bigint Group::count_ms(int igroup)
+{
+  int groupbit = bitmask[igroup];
+
+  int *mask = atom->mask;
+  int nlocal = atom->nlocal;
+
+  int n = 0;
+  if (fix_ms_) {
+    double ntmp = 0; // for ratios of a multisphere; cast afterwards to int
+    for (int i = 0; i < nlocal; i++)
+      if (mask[i] & groupbit) ntmp+=fix_ms_->get_volumeweight()->vector_atom[i];
+
+    n = static_cast<int>(ntmp);
+  } else {
+    for (int i = 0; i < nlocal; i++)
+      if (mask[i] & groupbit) n++;
+  }
+
+  bigint nsingle = n;
+  bigint nall;
+  MPI_Allreduce(&nsingle,&nall,1,MPI_LMP_BIGINT,MPI_SUM,world);
+  return nall;
+}
+
+/* ----------------------------------------------------------------------
+   count atoms in group and region
+------------------------------------------------------------------------- */
+
+bigint Group::count_ms(int igroup, int iregion)
+{
+  int groupbit = bitmask[igroup];
+  Region *region = domain->regions[iregion];
+
+  double **x = atom->x;
+  int *mask = atom->mask;
+  int nlocal = atom->nlocal;
+
+  int n = 0;
+  if (fix_ms_) {
+    double ntmp = 0; // for ratios of a multisphere; cast afterwards to int
+    for (int i = 0; i < nlocal; i++)
+      if (mask[i] & groupbit && region->match(x[i][0],x[i][1],x[i][2]))
+        ntmp+=fix_ms_->get_volumeweight()->vector_atom[i];
+
+    n = static_cast<int>(ntmp);
+  } else {
+    for (int i = 0; i < nlocal; i++)
+      if (mask[i] & groupbit && region->match(x[i][0],x[i][1],x[i][2])) n++;
+  }
+
+  bigint nsingle = n;
+  bigint nall;
+  MPI_Allreduce(&nsingle,&nall,1,MPI_LMP_BIGINT,MPI_SUM,world);
+  return nall;
+}
+
 /* ----------------------------------------------------------------------
    compute the total mass of group of atoms
    use either per-type mass or per-atom rmass
@@ -642,12 +714,24 @@ double Group::mass(int igroup)
 
   double one = 0.0;
 
-  if (rmass) {
-    for (int i = 0; i < nlocal; i++)
-      if (mask[i] & groupbit) one += rmass[i];
+  if (fix_ms_) {
+    if (rmass) {
+      for (int i = 0; i < nlocal; i++) {
+        
+        if (mask[i] & groupbit) one += rmass[i]*fix_ms_->get_volumeweight()->vector_atom[i];
+      }
+    } else {
+      for (int i = 0; i < nlocal; i++)
+        if (mask[i] & groupbit) one += mass[type[i]]*fix_ms_->get_volumeweight()->vector_atom[i];
+    }
   } else {
-    for (int i = 0; i < nlocal; i++)
-      if (mask[i] & groupbit) one += mass[type[i]];
+    if (rmass) {
+      for (int i = 0; i < nlocal; i++)
+        if (mask[i] & groupbit) one += rmass[i];
+    } else {
+      for (int i = 0; i < nlocal; i++)
+        if (mask[i] & groupbit) one += mass[type[i]];
+    }
   }
 
   double all;
@@ -674,14 +758,26 @@ double Group::mass(int igroup, int iregion)
 
   double one = 0.0;
 
-  if (rmass) {
-    for (int i = 0; i < nlocal; i++)
-      if (mask[i] & groupbit && region->match(x[i][0],x[i][1],x[i][2]))
-        one += rmass[i];
+  if (fix_ms_) {
+    if (rmass) {
+      for (int i = 0; i < nlocal; i++)
+        if (mask[i] & groupbit && region->match(x[i][0],x[i][1],x[i][2]))
+          one += rmass[i]*fix_ms_->get_volumeweight()->vector_atom[i];
+    } else {
+      for (int i = 0; i < nlocal; i++)
+        if (mask[i] & groupbit && region->match(x[i][0],x[i][1],x[i][2]))
+          one += mass[type[i]]*fix_ms_->get_volumeweight()->vector_atom[i];
+    }
   } else {
-    for (int i = 0; i < nlocal; i++)
-      if (mask[i] & groupbit && region->match(x[i][0],x[i][1],x[i][2]))
-        one += mass[type[i]];
+    if (rmass) {
+      for (int i = 0; i < nlocal; i++)
+        if (mask[i] & groupbit && region->match(x[i][0],x[i][1],x[i][2]))
+          one += rmass[i];
+    } else {
+      for (int i = 0; i < nlocal; i++)
+        if (mask[i] & groupbit && region->match(x[i][0],x[i][1],x[i][2]))
+          one += mass[type[i]];
+    }
   }
 
   double all;
@@ -702,8 +798,13 @@ double Group::charge(int igroup)
   int nlocal = atom->nlocal;
 
   double qone = 0.0;
-  for (int i = 0; i < nlocal; i++)
-    if (mask[i] & groupbit) qone += q[i];
+  if (fix_ms_) {
+    for (int i = 0; i < nlocal; i++)
+      if (mask[i] & groupbit) qone += q[i] * fix_ms_->get_volumeweight()->vector_atom[i];
+  } else {
+    for (int i = 0; i < nlocal; i++)
+      if (mask[i] & groupbit) qone += q[i];
+  }
 
   double qall;
   MPI_Allreduce(&qone,&qall,1,MPI_DOUBLE,MPI_SUM,world);
@@ -725,9 +826,15 @@ double Group::charge(int igroup, int iregion)
   int nlocal = atom->nlocal;
 
   double qone = 0.0;
-  for (int i = 0; i < nlocal; i++)
-    if (mask[i] & groupbit && region->match(x[i][0],x[i][1],x[i][2]))
-      qone += q[i];
+  if (fix_ms_) {
+    for (int i = 0; i < nlocal; i++)
+      if (mask[i] & groupbit && region->match(x[i][0],x[i][1],x[i][2]))
+        qone += q[i] * fix_ms_->get_volumeweight()->vector_atom[i];
+  } else {
+    for (int i = 0; i < nlocal; i++)
+      if (mask[i] & groupbit && region->match(x[i][0],x[i][1],x[i][2]))
+        qone += q[i];
+  }
 
   double qall;
   MPI_Allreduce(&qone,&qall,1,MPI_DOUBLE,MPI_SUM,world);
@@ -845,11 +952,11 @@ void Group::xcm(int igroup, double masstotal, double *cm)
 
   double massone;
   double unwrap[3];
-
   if (rmass) {
     for (int i = 0; i < nlocal; i++)
       if (mask[i] & groupbit) {
         massone = rmass[i];
+        if (fix_ms_) massone *= fix_ms_->get_volumeweight()->vector_atom[i];
         domain->unmap(x[i],image[i],unwrap);
         cmone[0] += unwrap[0] * massone;
         cmone[1] += unwrap[1] * massone;
@@ -859,6 +966,7 @@ void Group::xcm(int igroup, double masstotal, double *cm)
     for (int i = 0; i < nlocal; i++)
       if (mask[i] & groupbit) {
         massone = mass[type[i]];
+        if (fix_ms_) massone *= fix_ms_->get_volumeweight()->vector_atom[i];
         domain->unmap(x[i],image[i],unwrap);
         cmone[0] += unwrap[0] * massone;
         cmone[1] += unwrap[1] * massone;
@@ -899,11 +1007,11 @@ void Group::xcm(int igroup, double masstotal, double *cm, int iregion)
 
   double massone;
   double unwrap[3];
-
   if (rmass) {
     for (int i = 0; i < nlocal; i++)
       if (mask[i] & groupbit && region->match(x[i][0],x[i][1],x[i][2])) {
         massone = rmass[i];
+        if (fix_ms_) massone *= fix_ms_->get_volumeweight()->vector_atom[i];
         domain->unmap(x[i],image[i],unwrap);
         cmone[0] += unwrap[0] * massone;
         cmone[1] += unwrap[1] * massone;
@@ -913,6 +1021,7 @@ void Group::xcm(int igroup, double masstotal, double *cm, int iregion)
     for (int i = 0; i < nlocal; i++)
       if (mask[i] & groupbit && region->match(x[i][0],x[i][1],x[i][2])) {
         massone = mass[type[i]];
+        if (fix_ms_) massone *= fix_ms_->get_volumeweight()->vector_atom[i];
         domain->unmap(x[i],image[i],unwrap);
         cmone[0] += unwrap[0] * massone;
         cmone[1] += unwrap[1] * massone;
@@ -947,11 +1056,11 @@ void Group::vcm(int igroup, double masstotal, double *cm)
 
   double p[3],massone;
   p[0] = p[1] = p[2] = 0.0;
-
   if (rmass) {
     for (int i = 0; i < nlocal; i++)
       if (mask[i] & groupbit) {
         massone = rmass[i];
+        if (fix_ms_) massone *= fix_ms_->get_volumeweight()->vector_atom[i];
         p[0] += v[i][0]*massone;
         p[1] += v[i][1]*massone;
         p[2] += v[i][2]*massone;
@@ -960,6 +1069,7 @@ void Group::vcm(int igroup, double masstotal, double *cm)
     for (int i = 0; i < nlocal; i++)
       if (mask[i] & groupbit) {
         massone = mass[type[i]];
+        if (fix_ms_) massone *= fix_ms_->get_volumeweight()->vector_atom[i];
         p[0] += v[i][0]*massone;
         p[1] += v[i][1]*massone;
         p[2] += v[i][2]*massone;
@@ -995,11 +1105,11 @@ void Group::vcm(int igroup, double masstotal, double *cm, int iregion)
 
   double p[3],massone;
   p[0] = p[1] = p[2] = 0.0;
-
   if (rmass) {
     for (int i = 0; i < nlocal; i++)
       if (mask[i] & groupbit && region->match(x[i][0],x[i][1],x[i][2])) {
         massone = rmass[i];
+        if (fix_ms_) massone *= fix_ms_->get_volumeweight()->vector_atom[i];
         p[0] += v[i][0]*massone;
         p[1] += v[i][1]*massone;
         p[2] += v[i][2]*massone;
@@ -1008,6 +1118,7 @@ void Group::vcm(int igroup, double masstotal, double *cm, int iregion)
     for (int i = 0; i < nlocal; i++)
       if (mask[i] & groupbit && region->match(x[i][0],x[i][1],x[i][2])) {
         massone = mass[type[i]];
+        if (fix_ms_) massone *= fix_ms_->get_volumeweight()->vector_atom[i];
         p[0] += v[i][0]*massone;
         p[1] += v[i][1]*massone;
         p[2] += v[i][2]*massone;
@@ -1091,16 +1202,30 @@ double Group::ke(int igroup)
 
   double one = 0.0;
 
-  if (rmass) {
-    for (int i = 0; i < nlocal; i++)
-      if (mask[i] & groupbit)
-        one += (v[i][0]*v[i][0] + v[i][1]*v[i][1] + v[i][2]*v[i][2]) *
-          rmass[i];
+  if (fix_ms_) {
+    if (rmass) {
+      for (int i = 0; i < nlocal; i++)
+        if (mask[i] & groupbit)
+          one += (v[i][0]*v[i][0] + v[i][1]*v[i][1] + v[i][2]*v[i][2]) *
+              rmass[i] * fix_ms_->get_volumeweight()->vector_atom[i];
+    } else {
+      for (int i = 0; i < nlocal; i++)
+        if (mask[i] & groupbit)
+          one += (v[i][0]*v[i][0] + v[i][1]*v[i][1] + v[i][2]*v[i][2]) *
+              mass[type[i]] * fix_ms_->get_volumeweight()->vector_atom[i];
+    }
   } else {
-    for (int i = 0; i < nlocal; i++)
-      if (mask[i] & groupbit)
-        one += (v[i][0]*v[i][0] + v[i][1]*v[i][1] + v[i][2]*v[i][2]) *
-          mass[type[i]];
+    if (rmass) {
+      for (int i = 0; i < nlocal; i++)
+        if (mask[i] & groupbit)
+          one += (v[i][0]*v[i][0] + v[i][1]*v[i][1] + v[i][2]*v[i][2]) *
+              rmass[i];
+    } else {
+      for (int i = 0; i < nlocal; i++)
+        if (mask[i] & groupbit)
+          one += (v[i][0]*v[i][0] + v[i][1]*v[i][1] + v[i][2]*v[i][2]) *
+              mass[type[i]];
+    }
   }
 
   double all;
@@ -1128,16 +1253,30 @@ double Group::ke(int igroup, int iregion)
 
   double one = 0.0;
 
-  if (rmass) {
-    for (int i = 0; i < nlocal; i++)
-      if (mask[i] & groupbit && region->match(x[i][0],x[i][1],x[i][2]))
-        one += (v[i][0]*v[i][0] + v[i][1]*v[i][1] + v[i][2]*v[i][2]) *
-          rmass[i];
+  if (fix_ms_) {
+    if (rmass) {
+      for (int i = 0; i < nlocal; i++)
+        if (mask[i] & groupbit && region->match(x[i][0],x[i][1],x[i][2]))
+          one += (v[i][0]*v[i][0] + v[i][1]*v[i][1] + v[i][2]*v[i][2]) *
+              rmass[i] * fix_ms_->get_volumeweight()->vector_atom[i];
+    } else {
+      for (int i = 0; i < nlocal; i++)
+        if (mask[i] & groupbit && region->match(x[i][0],x[i][1],x[i][2]))
+          one += (v[i][0]*v[i][0] + v[i][1]*v[i][1] + v[i][2]*v[i][2]) *
+              mass[type[i]] * fix_ms_->get_volumeweight()->vector_atom[i];
+    }
   } else {
-    for (int i = 0; i < nlocal; i++)
-      if (mask[i] & groupbit && region->match(x[i][0],x[i][1],x[i][2]))
-        one += (v[i][0]*v[i][0] + v[i][1]*v[i][1] + v[i][2]*v[i][2]) *
-          mass[type[i]];
+    if (rmass) {
+      for (int i = 0; i < nlocal; i++)
+        if (mask[i] & groupbit && region->match(x[i][0],x[i][1],x[i][2]))
+          one += (v[i][0]*v[i][0] + v[i][1]*v[i][1] + v[i][2]*v[i][2]) *
+              rmass[i];
+    } else {
+      for (int i = 0; i < nlocal; i++)
+        if (mask[i] & groupbit && region->match(x[i][0],x[i][1],x[i][2]))
+          one += (v[i][0]*v[i][0] + v[i][1]*v[i][1] + v[i][2]*v[i][2]) *
+              mass[type[i]];
+    }
   }
 
   double all;
@@ -1167,7 +1306,6 @@ double Group::gyration(int igroup, double masstotal, double *cm)
   double dx,dy,dz,massone;
   double unwrap[3];
   double rg = 0.0;
-
   for (int i = 0; i < nlocal; i++)
     if (mask[i] & groupbit) {
       domain->unmap(x[i],image[i],unwrap);
@@ -1176,6 +1314,7 @@ double Group::gyration(int igroup, double masstotal, double *cm)
       dz = unwrap[2] - cm[2];
       if (rmass) massone = rmass[i];
       else massone = mass[type[i]];
+      if (fix_ms_) massone *= fix_ms_->get_volumeweight()->vector_atom[i];
       rg += (dx*dx + dy*dy + dz*dz) * massone;
     }
   double rg_all;
@@ -1207,7 +1346,6 @@ double Group::gyration(int igroup, double masstotal, double *cm, int iregion)
   double dx,dy,dz,massone;
   double unwrap[3];
   double rg = 0.0;
-
   for (int i = 0; i < nlocal; i++)
     if (mask[i] & groupbit && region->match(x[i][0],x[i][1],x[i][2])) {
       domain->unmap(x[i],image[i],unwrap);
@@ -1216,6 +1354,7 @@ double Group::gyration(int igroup, double masstotal, double *cm, int iregion)
       dz = unwrap[2] - cm[2];
       if (rmass) massone = rmass[i];
       else massone = mass[type[i]];
+      if (fix_ms_) massone *= fix_ms_->get_volumeweight()->vector_atom[i];
       rg += (dx*dx + dy*dy + dz*dz) * massone;
     }
   double rg_all;
@@ -1249,7 +1388,6 @@ void Group::angmom(int igroup, double *cm, double *lmom)
 
   double p[3];
   p[0] = p[1] = p[2] = 0.0;
-
   for (int i = 0; i < nlocal; i++)
     if (mask[i] & groupbit) {
       domain->unmap(x[i],image[i],unwrap);
@@ -1258,6 +1396,7 @@ void Group::angmom(int igroup, double *cm, double *lmom)
       dz = unwrap[2] - cm[2];
       if (rmass) massone = rmass[i];
       else massone = mass[type[i]];
+      if (fix_ms_) massone *= fix_ms_->get_volumeweight()->vector_atom[i];
       p[0] += massone * (dy*v[i][2] - dz*v[i][1]);
       p[1] += massone * (dz*v[i][0] - dx*v[i][2]);
       p[2] += massone * (dx*v[i][1] - dy*v[i][0]);
@@ -1300,6 +1439,7 @@ void Group::angmom(int igroup, double *cm, double *lmom, int iregion)
       dz = unwrap[2] - cm[2];
       if (rmass) massone = rmass[i];
       else massone = mass[type[i]];
+      if (fix_ms_) massone *= fix_ms_->get_volumeweight()->vector_atom[i];
       p[0] += massone * (dy*v[i][2] - dz*v[i][1]);
       p[1] += massone * (dz*v[i][0] - dx*v[i][2]);
       p[2] += massone * (dx*v[i][1] - dy*v[i][0]);
@@ -1416,6 +1556,7 @@ void Group::inertia(int igroup, double *cm, double itensor[3][3])
       dz = unwrap[2] - cm[2];
       if (rmass) massone = rmass[i];
       else massone = mass[type[i]];
+      if (fix_ms_) massone *= fix_ms_->get_volumeweight()->vector_atom[i];
       ione[0][0] += massone * (dy*dy + dz*dz);
       ione[1][1] += massone * (dx*dx + dz*dz);
       ione[2][2] += massone * (dx*dx + dy*dy);
@@ -1466,6 +1607,7 @@ void Group::inertia(int igroup, double *cm, double itensor[3][3], int iregion)
       dz = unwrap[2] - cm[2];
       if (rmass) massone = rmass[i];
       else massone = mass[type[i]];
+      if (fix_ms_) massone *= fix_ms_->get_volumeweight()->vector_atom[i];
       ione[0][0] += massone * (dy*dy + dz*dz);
       ione[1][1] += massone * (dx*dx + dz*dz);
       ione[2][2] += massone * (dx*dx + dy*dy);
