@@ -83,6 +83,7 @@
 #include "accelerator_cuda.h"
 #include "error.h"
 #include "memory.h"
+#include "signal_handling.h"
 
 #ifdef _OPENMP
 #include "omp.h"
@@ -257,6 +258,9 @@ void Input::file()
       sprintf(str,"Unknown command: %s",line);
       error->all(FLERR,str);
     }
+
+    if (SignalHandler::request_quit)
+        return;
   }
 }
 
@@ -597,7 +601,6 @@ int Input::execute_command()
   else if (!strcmp(command,"mass")) mass();
   else if (!strcmp(command,"min_modify")) min_modify();
   else if (!strcmp(command,"min_style")) min_style();
-  else if (!strcmp(command,"neigh_modify")) neigh_modify();
   else if (!strcmp(command,"neighbor")) neighbor_command();
   else if (!strcmp(command,"newton")) newton();
   else if (!strcmp(command,"package")) package();
@@ -611,6 +614,8 @@ int Input::execute_command()
   else if (!strcmp(command,"restart")) restart();
   else if (!strcmp(command,"run_style")) run_style();
   else if (!strcmp(command,"soft_particles")) soft_particles(); 
+  else if (!strcmp(command,"hard_particles")) hard_particles();
+  else if (!strcmp(command,"write_restart_on_signal")) write_restart_on_signal();
   else if (!strcmp(command,"special_bonds")) special_bonds();
   else if (!strcmp(command,"suffix")) suffix();
   else if (!strcmp(command,"thermo")) thermo();
@@ -751,7 +756,12 @@ void Input::ifthenelse()
     }
 
     ifthenelse_flag = 1;
-    for (int i = 0; i < ncommands; i++) one(commands[i]);
+    for (int i = 0; i < ncommands; i++)
+    {
+        one(commands[i]);
+        if (SignalHandler::request_quit)
+            break;
+    }
     ifthenelse_flag = 0;
 
     for (int i = 0; i < ncommands; i++) delete [] commands[i];
@@ -861,7 +871,12 @@ void Input::jump()
   }
 
   if (me == 0) {
-    if (strcmp(arg[0],"SELF") == 0) rewind(infile);
+    if (strcmp(arg[0],"SELF") == 0)
+    {
+        if (infile == stdin)
+            error->one(FLERR, "jump SELF is not allowed when using stdin, use -in instead");
+        rewind(infile);
+    }
     else {
       if (infile != stdin) fclose(infile);
       infile = fopen(arg[0],"r");
@@ -1398,16 +1413,9 @@ void Input::min_style()
 
 /* ---------------------------------------------------------------------- */
 
-void Input::neigh_modify()
-{
-  neighbor->modify_params(narg,arg);
-}
-
-/* ---------------------------------------------------------------------- */
-
 void Input::neighbor_command()
 {
-  neighbor->set(narg,arg);
+  neighbor->set(narg, arg);
 }
 
 /* ---------------------------------------------------------------------- */
@@ -1584,6 +1592,32 @@ void Input::run_style()
   update->create_integrate(narg,arg,lmp->suffix);
 }
 
+/* ---------------------------------------------------------------------- */
+void Input::write_restart_on_signal()
+{
+   if(narg != 1)
+      error->all(FLERR,"write_restart_on_signal expects 'yes' or 'no'");
+   if(0 == strcmp(arg[0],"yes"))
+      SignalHandler::enable_restart_writing = true;
+   else if(0 == strcmp(arg[0],"no"))
+      SignalHandler::enable_restart_writing = false;
+   else
+      error->all(FLERR,"write_restart_on_signal expects 'yes' or 'no'");
+}
+
+/* ---------------------------------------------------------------------- */
+void Input::hard_particles()
+{
+   if(narg != 1)
+      error->all(FLERR,"hard_particles expects 'yes' or 'no'");
+   if(0 == strcmp(arg[0],"yes"))
+      atom->get_properties()->do_allow_hard_particles();
+   else if(0 == strcmp(arg[0],"no"))
+      atom->get_properties()->do_not_allow_hard_particles();
+   else
+      error->all(FLERR,"hard_particles expects 'yes' or 'no'");
+}
+
 /* ---------------------------------------------------------------------- */
 
 void Input::soft_particles()
@@ -1595,7 +1629,7 @@ void Input::soft_particles()
    else if(0 == strcmp(arg[0],"no"))
       atom->get_properties()->do_not_allow_soft_particles();
    else
-      error->all(FLERR,"Soft_particles expects 'yes' or 'no'");
+      error->all(FLERR,"soft_particles expects 'yes' or 'no'");
 }
 
 /* ---------------------------------------------------------------------- */
@@ -1673,6 +1707,7 @@ void Input::timestep()
 {
   if (narg != 1) error->all(FLERR,"Illegal timestep command");
   update->dt = force->numeric(FLERR,arg[0]);
+  update->timestep_set = true;
   
 }
 
