@@ -116,7 +116,7 @@ void lammps_close(void *ptr)
    process an input script in filename str
 ------------------------------------------------------------------------- */
 
-void lammps_file(void *ptr, char *str)
+void lammps_file(void *ptr, const char *str)
 {
   LAMMPS *lmp = (LAMMPS *) ptr;
   lmp->input->file(str);
@@ -126,7 +126,7 @@ void lammps_file(void *ptr, char *str)
    process a single input command in str
 ------------------------------------------------------------------------- */
 
-char *lammps_command(void *ptr, char *str)
+char *lammps_command(void *ptr, const char *str)
 {
   LAMMPS *lmp = (LAMMPS *) ptr;
   return lmp->input->one(str);
@@ -156,7 +156,7 @@ void lammps_free(void *ptr)
    customize by adding names
 ------------------------------------------------------------------------- */
 
-void *lammps_extract_global(void *ptr, char *name)
+void *lammps_extract_global(void *ptr, const char *name)
 {
   LAMMPS *lmp = (LAMMPS *) ptr;
 
@@ -189,6 +189,8 @@ void *lammps_extract_global(void *ptr, char *name)
   if (strcmp(name,"nlocal") == 0) return (void *) &lmp->atom->nlocal;
   if (strcmp(name,"nghost") == 0) return (void *) &lmp->atom->nghost;
   if (strcmp(name,"ago") == 0) return (void *) &lmp->neighbor->ago; //INT,  time steps since last neighbor->decide,
+  if (strcmp(name,"ntimestep") == 0) return (void *) &lmp->update->ntimestep;
+  if (strcmp(name,"firststep") == 0) return (void *) &lmp->update->firststep;
   return NULL;
 }
 
@@ -201,7 +203,7 @@ void *lammps_extract_global(void *ptr, char *name)
    customize by adding names to Atom::extract()
 ------------------------------------------------------------------------- */
 
-void *lammps_extract_atom(void *ptr, char *name)
+void *lammps_extract_atom(void *ptr, const char *name)
 {
   LAMMPS *lmp = (LAMMPS *) ptr;
   return lmp->atom->extract(name);
@@ -228,7 +230,7 @@ void *lammps_extract_atom(void *ptr, char *name)
      so caller must insure that it is OK
 ------------------------------------------------------------------------- */
 
-void *lammps_extract_compute(void *ptr, char *id, int style, int type)
+void *lammps_extract_compute(void *ptr, const char *id, int style, int type)
 {
   LAMMPS *lmp = (LAMMPS *) ptr;
 
@@ -312,7 +314,7 @@ void *lammps_extract_compute(void *ptr, char *id, int style, int type)
      the fix is valid, so caller must insure that it is OK
 ------------------------------------------------------------------------- */
 
-void *lammps_extract_fix(void *ptr, char *id, int style, int type,
+void *lammps_extract_fix(void *ptr, const char *id, int style, int type,
                          int i, int j)
 {
   LAMMPS *lmp = (LAMMPS *) ptr;
@@ -430,7 +432,7 @@ int lammps_get_natoms(void *ptr)
    data must be pre-allocated by caller to correct length
 ------------------------------------------------------------------------- */
 
-void lammps_gather_atoms(void *ptr, char *name,
+void lammps_gather_atoms(void *ptr, const char *name,
                          int type, int count, void *data)
 {
   LAMMPS *lmp = (LAMMPS *) ptr;
@@ -518,7 +520,7 @@ void lammps_gather_atoms(void *ptr, char *name,
      e.g. x[0][0],x[0][1],x[0][2],x[1][0],x[1][1],x[1][2],x[2][0],...
 ------------------------------------------------------------------------- */
 
-void lammps_scatter_atoms(void *ptr, char *name,
+void lammps_scatter_atoms(void *ptr, const char *name,
                           int type, int count, void *data)
 {
   LAMMPS *lmp = (LAMMPS *) ptr;
