@@ -52,6 +52,7 @@
 #include "container.h"
 #include "bounding_box.h"
 #include "random_park.h"
+#include <stdio.h>
 
 #define EPSILON_PRECISION 1e-8
 
@@ -79,18 +80,18 @@ namespace LAMMPS_NS
         virtual void scale(double factor);
 
         // linear move w/ total and incremental displacement
-        virtual void move(double *vecTotal, double *vecIncremental);
+        virtual void move(const double * const vecTotal, const double * const vecIncremental);
 
         // linear move w/ incremental displacement
-        virtual void move(double *vecIncremental);
+        virtual void move(const double * const vecIncremental);
 
         // rotation w/ total and incremental displacement
         //   calls rotate(double *totalQuat,double *dQuat,double *displacement)
-        void rotate(double totalAngle, double dAngle, double *axis, double *p);
+        void rotate(const double totalAngle, const double dAngle, const double * const axis, const double * const p);
 
         // rotation w/ incremental displacement
         //   calls rotate(double *dQuat,double *displacement)
-        void rotate(double dAngle, double *axis, double *p);
+        void rotate(const double dAngle, const double * const axis, const double * const p);
 
         void updateCenterRbound(int ilo,int ihi);
 
@@ -98,6 +99,24 @@ namespace LAMMPS_NS
         bool registerMove(bool _scale, bool _translate, bool _rotate);
         void unregisterMove(bool _scale, bool _translate, bool _rotate);
 
+        // flags for saving global linear and angular velocity
+        void set_store_vel()
+        { store_vel++; }
+        void set_store_omega()
+        { store_omega++; }
+        void unset_store_vel()
+        { store_vel = store_vel > 1 ? store_vel-1 : 0; }
+        void unset_store_omega()
+        { store_omega = store_omega > 1 ? store_omega-1 : 0; }
+        bool is_set_store_vel()
+        { return store_vel ? true : false; }
+        bool is_set_store_omega()
+        { return store_omega ? true : false; }
+
+        // functions to obtain global linear and angular velocity
+        void get_global_vel(double * vel);
+        void get_global_omega(double * omega);
+
         // bbox stuff
         BoundingBox getGlobalBoundingBox() const;
         BoundingBox getElementBoundingBoxOnSubdomain(int const n);
@@ -182,12 +201,12 @@ namespace LAMMPS_NS
         void extendToElem(int const nElem) const;
 
         // linear move of single element w/ incremental displacement
-        virtual void moveElement(int i,double *vecIncremental);
+        virtual void moveElement(const int i, const double * const vecIncremental);
 
         // rotation using quaternions
         
-        virtual void rotate(double *totalQ, double *dQ,double *origin);
-        virtual void rotate(double *dQ, double *origin);
+        virtual void rotate(const double * const totalQ, const double * const dQ, const double * const origin);
+        virtual void rotate(const double * const dQ, const double * const origin);
 
         // mesh nodes
         MultiVectorContainer<double,NUM_NODES,3> node_;
@@ -248,6 +267,15 @@ namespace LAMMPS_NS
         int nMove_;
         int nScale_,nTranslate_,nRotate_;
 
+        // counters to decide whether mesh linear and angular velocity are stored
+        int store_vel, store_omega;
+
+        // step when velocities where last stored
+        int step_store_vel, step_store_omega;
+
+        // storage for global mesh linear and angular velocity
+        double global_vel[3], global_quaternion[4], prev_quaternion[4];
+
         // store current node position for use by moving mesh
         void storeNodePosOrig(int ilo, int ihi);
 
