@@ -68,8 +68,17 @@
     nScale_(0),
     nTranslate_(0),
     nRotate_(0),
+    store_vel(0),
+    store_omega(0),
+    step_store_vel(0),
+    step_store_omega(0),
     stepLastReset_(-1)
   {
+    vectorZeroize3D(global_vel);
+    quatIdentity4D(global_quaternion);
+    quatIdentity4D(prev_quaternion);
+    center_.setWrapPeriodic(true);
+    node_.setWrapPeriodic(true);
   }
 
   /* ----------------------------------------------------------------------
@@ -171,6 +180,7 @@
                 
                 node_.del(n);
                 center_.del(n);
+                rBound_.del(n);
                 return false;
             }
         }
@@ -363,6 +373,7 @@
             error->one(FLERR,"Illegal situation in MultiNodeMesh<NUM_NODES>::registerMove");
 
           node_orig_ = new MultiVectorContainer<double,NUM_NODES,3>("node_orig");
+          node_orig_->setWrapPeriodic(true);
           for(int i = 0; i < nall; i++)
           {
             for(int j = 0; j < NUM_NODES; j++)
@@ -449,14 +460,14 @@
   ------------------------------------------------------------------------- */
 
   template<int NUM_NODES>
-  void MultiNodeMesh<NUM_NODES>::move(double *vecTotal, double *vecIncremental)
+  void MultiNodeMesh<NUM_NODES>::move(const double * const vecTotal, const double * const vecIncremental)
   {
     if(!isTranslating())
         this->error->all(FLERR,"Illegal call, need to register movement first");
 
-    resetToOrig();
+    const int n = sizeLocal() + sizeGhost();
 
-    int n = sizeLocal() + sizeGhost();
+    resetToOrig();
 
     for(int i = 0; i < n; i++)
     {
@@ -470,6 +481,16 @@
         vectorScalarDiv3D(center_(i),static_cast<double>(NUM_NODES));
     }
 
+    if (store_vel)
+    {
+        if (step_store_vel != update->ntimestep)
+        {
+            step_store_vel = update->ntimestep;
+            vectorZeroize3D(global_vel);
+        }
+        vectorAddMultiple3D(global_vel, 1.0/update->dt, vecIncremental, global_vel);
+    }
+
     updateGlobalBoundingBox();
   }
 
@@ -478,7 +499,7 @@
   ------------------------------------------------------------------------- */
 
   template<int NUM_NODES>
-  void MultiNodeMesh<NUM_NODES>::move(double *vecIncremental)
+  void MultiNodeMesh<NUM_NODES>::move(const double * const vecIncremental)
   {
     
     int n = sizeLocal() + sizeGhost();
@@ -491,6 +512,16 @@
         vectorAdd3D(center_(i),vecIncremental,center_(i));
     }
 
+    if (store_vel)
+    {
+        if (step_store_vel != update->ntimestep)
+        {
+            step_store_vel = update->ntimestep;
+            vectorZeroize3D(global_vel);
+        }
+        vectorAddMultiple3D(global_vel, 1.0/update->dt, vecIncremental, global_vel);
+    }
+
     updateGlobalBoundingBox();
   }
   /* ----------------------------------------------------------------------
@@ -498,7 +529,7 @@
   ------------------------------------------------------------------------- */
 
   template<int NUM_NODES>
-  void MultiNodeMesh<NUM_NODES>::moveElement(int i,double *vecIncremental)
+  void MultiNodeMesh<NUM_NODES>::moveElement(const int i, const double * const vecIncremental)
   {
     for(int j = 0; j < NUM_NODES; j++)
             vectorAdd3D(node_(i)[j],vecIncremental,node_(i)[j]);
@@ -514,7 +545,7 @@
   ------------------------------------------------------------------------- */
 
   template<int NUM_NODES>
-  void MultiNodeMesh<NUM_NODES>::rotate(double totalAngle, double dAngle, double *axis, double *p)
+  void MultiNodeMesh<NUM_NODES>::rotate(const double totalAngle, const double dAngle, const double * const axis, const double * const p)
   {
     double totalQ[4],dQ[4], axisNorm[3], origin[3];
 
@@ -542,7 +573,7 @@
   }
 
   template<int NUM_NODES>
-  void MultiNodeMesh<NUM_NODES>::rotate(double *totalQ, double *dQ,double *origin)
+  void MultiNodeMesh<NUM_NODES>::rotate(const double * const totalQ, const double * const dQ, const double * const origin)
   {
     if(!isRotating())
         this->error->all(FLERR,"Illegal call, need to register movement first");
@@ -568,6 +599,16 @@
       vectorScalarDiv3D(center_(i),static_cast<double>(NUM_NODES));
     }
 
+    if (store_omega)
+    {
+        if (step_store_omega != update->ntimestep)
+        {
+            step_store_omega = update->ntimestep;
+            vectorCopy4D(global_quaternion, prev_quaternion);
+        }
+        vectorCopy4D(totalQ, global_quaternion);
+    }
+
     updateGlobalBoundingBox();
   }
 
@@ -576,7 +617,7 @@
   ------------------------------------------------------------------------- */
 
   template<int NUM_NODES>
-  void MultiNodeMesh<NUM_NODES>::rotate(double dAngle, double *axis, double *p)
+  void MultiNodeMesh<NUM_NODES>::rotate(const double dAngle, const double * const axis, const double * const p)
   {
     double dQ[4], axisNorm[3], origin[3];
 
@@ -599,7 +640,7 @@
   }
 
   template<int NUM_NODES>
-  void MultiNodeMesh<NUM_NODES>::rotate(double *dQ, double *origin)
+  void MultiNodeMesh<NUM_NODES>::rotate(const double * const dQ, const double * const origin)
   {
     
     int n = sizeLocal() + sizeGhost();
@@ -621,6 +662,16 @@
       vectorScalarDiv3D(center_(i),static_cast<double>(NUM_NODES));
     }
 
+    if (store_omega)
+    {
+        if (step_store_omega != update->ntimestep)
+        {
+            step_store_omega = update->ntimestep;
+            vectorCopy4D(global_quaternion, prev_quaternion);
+        }
+        quatMult4D(global_quaternion, dQ);
+    }
+
     updateGlobalBoundingBox();
   }
 
@@ -839,4 +890,43 @@
 
   }
 
+template<int NUM_NODES>
+void MultiNodeMesh<NUM_NODES>::get_global_vel(double *vel)
+{
+    if (!store_vel)
+        return;
+    if (step_store_vel != update->ntimestep)
+    {
+        step_store_vel = update->ntimestep;
+        vectorZeroize3D(global_vel);
+    }
+    vectorCopy3D(global_vel, vel);
+}
+
+template<int NUM_NODES>
+void MultiNodeMesh<NUM_NODES>::get_global_omega(double *omega)
+{
+    if (!store_omega)
+        return;
+    if (step_store_omega != update->ntimestep)
+    {
+        step_store_omega = update->ntimestep;
+        vectorCopy4D(global_quaternion, prev_quaternion);
+    }
+    double dQ[4];
+    double invPrevQ[4];
+    quatInverse4D(prev_quaternion, invPrevQ);
+    quatMult4D(invPrevQ, global_quaternion, dQ);
+    dQ[0] = fmax(-1.0, fmin(1.0, dQ[0]));
+    const double dAngle = 2.0*acos(dQ[0]);
+    if (fabs(dAngle) > 1e-12)
+    {
+        const double SinHalfdAngle = sin(dAngle*0.5);
+        const double multi = dAngle/(SinHalfdAngle*update->dt);
+        vectorScalarMult3D(&(dQ[1]), multi, omega);
+    }
+    else
+        vectorZeroize3D(omega);
+}
+
 #endif
