@@ -57,6 +57,7 @@
 #include "atom.h"
 #include "atom_vec.h"
 #include "domain.h"
+#include "modify.h"
 #include "force.h"
 #include "update.h"
 #include "modify.h"
@@ -115,12 +116,16 @@ PairGran::PairGran(LAMMPS *lmp) : Pair(lmp)
 
   fix_contact_forces_ = NULL;
   store_contact_forces_ = false;
+  store_contact_forces_every_ = 1;
   fix_contact_forces_stress_ = NULL;
   store_contact_forces_stress_ = false;
   fix_store_multicontact_data_ = NULL;
   store_multicontact_data_ = false;
 
   fix_relax_ = NULL;
+
+  if(modify->n_fixes_style("multisphere/advanced"))
+    do_store_contact_forces();
 }
 
 /* ---------------------------------------------------------------------- */
@@ -494,6 +499,12 @@ void PairGran::init_style()
     delete []fixarg;
   }
 
+  fix_sum_normal_force_ =
+      static_cast<FixPropertyAtom*>
+      (
+          modify->find_fix_property("sum_normal_force_","property/atom","scalar",0,0, "pair/gran", false)
+      );
+
   // need a gran neigh list and optionally a granular history neigh list
 
   if(needs_neighlist)
@@ -783,3 +794,19 @@ double PairGran::memory_usage()
 bool PairGran::forceoff() {
   return false;
 }
+
+void PairGran::cpl_add_pair(LCM::SurfacesIntersectData & sidata, LCM::ForceData & i_forces)
+{
+    const double fx = i_forces.delta_F[0];
+    const double fy = i_forces.delta_F[1];
+    const double fz = i_forces.delta_F[2];
+    const double tor1 = i_forces.delta_torque[0];
+    const double tor2 = i_forces.delta_torque[1];
+    const double tor3 = i_forces.delta_torque[2];
+    cpl_->add_pair(sidata.i, sidata.j, fx,fy,fz,tor1,tor2,tor3,sidata.contact_history);
+}
+
+void PairGran::cpl_pair_finalize()
+{
+    cpl_->pair_finalize();
+}
