@@ -53,6 +53,7 @@
 #include "compute_pair_gran_local.h"
 #include "contact_interface.h"
 #include "fix_relax_contacts.h"
+#include "fix_property_atom.h"
 #include <vector>
 #include <string>
 
@@ -60,6 +61,8 @@ namespace LCM = LIGGGHTS::ContactModels;
 
 namespace LAMMPS_NS {
 
+class ComputePairGranLocal;
+
 class PairGran : public Pair, public LIGGGHTS::IContactHistorySetup {
 public:
 
@@ -97,19 +100,9 @@ public:
   void register_compute_pair_local(class ComputePairGranLocal *,int&);
   void unregister_compute_pair_local(class ComputePairGranLocal *ptr);
 
-  inline void cpl_add_pair(LCM::SurfacesIntersectData & sidata, LCM::ForceData & i_forces)
-  {
-    const double fx = i_forces.delta_F[0];
-    const double fy = i_forces.delta_F[1];
-    const double fz = i_forces.delta_F[2];
-    const double tor1 = i_forces.delta_torque[0];
-    const double tor2 = i_forces.delta_torque[1];
-    const double tor3 = i_forces.delta_torque[2];
-    cpl_->add_pair(sidata.i, sidata.j, fx,fy,fz,tor1,tor2,tor3,sidata.contact_history);
-  }
+  void cpl_add_pair(LCM::SurfacesIntersectData & sidata, LCM::ForceData & i_forces);
 
-  inline void cpl_pair_finalize()
-  { cpl_->pair_finalize(); }
+  void cpl_pair_finalize();
 
   /* PUBLIC ACCESS FUNCTIONS */
 
@@ -128,6 +121,9 @@ public:
   inline bool storeContactForces() const
   { return store_contact_forces_; }
 
+  inline int storeContactForcesEvery() const
+  { return store_contact_forces_every_; }
+
   inline bool storeContactForcesStress()
   { return store_contact_forces_stress_; }
 
@@ -176,9 +172,18 @@ public:
     return offset;
   }
 
+  void add_dissipated_energy(const double e)
+  { dissipated_energy_ += e; }
+
+  double get_dissipated_energy()
+  { return dissipated_energy_; }
+
   void do_store_contact_forces()
   { store_contact_forces_ = true; }
 
+  void do_store_contact_forces_every(int ev)
+  { store_contact_forces_ = true;  store_contact_forces_every_ = ev; }
+
   void do_relax_region(FixRelaxContacts *_fr)
   { fix_relax_ = _fr; }
 
@@ -188,6 +193,15 @@ public:
   void do_store_multicontact_data()
   { store_multicontact_data_ = true; }
 
+  class FixContactHistory* get_fix_history() const
+  { return fix_history; }
+
+  bool store_sum_normal_force() const
+  { return fix_sum_normal_force_ != NULL; }
+
+  double * get_sum_normal_force_ptr(const int i)
+  { return &(fix_sum_normal_force_->vector_atom[i]); }
+
  protected:
 
   struct HistoryArg {
@@ -234,6 +248,7 @@ public:
 
   // storage for per-contact forces and torque
   bool store_contact_forces_;
+  int store_contact_forces_every_;
   class FixContactPropertyAtom *fix_contact_forces_;
 
   // storage for per-contact forces and relative position (for goldhirsch stress model)
@@ -242,6 +257,8 @@ public:
   // storage for per contact delta data (for multicontact models)
   bool store_multicontact_data_;
   class FixContactPropertyAtom *fix_store_multicontact_data_;
+  // storage for simplistic pressure computation via normal forces
+  class FixPropertyAtom *fix_sum_normal_force_;
 
   // storage of rigid body masses for use in granular interactions
 
@@ -277,6 +294,9 @@ public:
   int *dnum_index;
 
   FixRelaxContacts *fix_relax_;
+
+  // dissipated energy in wall -> particle contacts
+  double dissipated_energy_;
 };
 
 }
