@@ -35,7 +35,8 @@
     (if not contributing author is listed, this file has been contributed
     by the core developer)
 
-    Christoph Kloss (DCS Computing GmbH, Linz, JKU Linz)
+    Christoph Kloss (DCS Computing GmbH, Linz)
+    Christoph Kloss (JKU Linz)
     Richard Berger (JKU Linz)
 
     Copyright 2012-     DCS Computing GmbH, Linz
@@ -70,6 +71,16 @@ int PropertyRegistry::max_type()
   return properties.max_type();
 }
 
+double PropertyRegistry::min_radius()
+{
+  return properties.min_radius();
+}
+
+double PropertyRegistry::max_radius()
+{
+  return properties.max_radius();
+}
+
 LAMMPS * PropertyRegistry::getLAMMPS()
 {
   return lmp;
@@ -153,6 +164,7 @@ void PropertyRegistry::connect(string varname, double ** & variable, const char
       matrices[varname] = (*matrix_creators[varname])(*this, caller, use_sanity_checks[varname]);
     } else {
       // ERROR unknown property
+      error->message(FLERR, "unknown matrix property");
     }
   }
   matrices[varname]->connect(variable);
@@ -165,6 +177,7 @@ void PropertyRegistry::connect(string varname, double * & variable, const char *
       vectors[varname] = (*vector_creators[varname])(*this, caller, use_sanity_checks[varname]);
     } else {
       // ERROR unknown property
+      error->message(FLERR, "unknown vector property");
     }
   }
   vectors[varname]->connect(variable);
@@ -177,6 +190,7 @@ void PropertyRegistry::connect(string varname, double & variable, const char *ca
       scalars[varname] = (*scalar_creators[varname])(*this, caller, use_sanity_checks[varname]);
     } else {
       // ERROR unknown property
+      error->message(FLERR, "unknown scalar property");
     }
   }
   scalars[varname]->connect(variable);
