@@ -44,9 +44,9 @@
 ------------------------------------------------------------------------- */
 
 #include "lmptype.h"
-#include "mpi.h"
-#include "string.h"
-#include "stdlib.h"
+#include <mpi.h>
+#include <string.h>
+#include <stdlib.h>
 //#include "sys/types.h"
 #include "dirent.h"
 #include "read_restart.h"
