@@ -47,6 +47,7 @@ ROLLING_MODEL(ROLLING_CDT,cdt,1)
 #ifndef ROLLING_MODEL_CDT_H_
 #define ROLLING_MODEL_CDT_H_
 #include "contact_models.h"
+#include "rolling_model_base.h"
 #include <algorithm>
 #include <math.h>
 #include "math_extra_liggghts.h"
@@ -57,17 +58,21 @@ namespace ContactModels
   using namespace LAMMPS_NS;
 
   template<>
-  class RollingModel<ROLLING_CDT> : protected Pointers {
+  class RollingModel<ROLLING_CDT> : public RollingModelBase {
   public:
-    static const int MASK = CM_CONNECT_TO_PROPERTIES | CM_SURFACES_INTERSECT;
-
-    RollingModel(LAMMPS * lmp, IContactHistorySetup*,class ContactModelBase *) :
-        Pointers(lmp), coeffRollFrict(NULL)
+    RollingModel(LAMMPS * lmp, IContactHistorySetup * hsetup, class ContactModelBase * c) :
+        RollingModelBase(lmp, hsetup, c), coeffRollFrict(NULL)
     {
       
     }
 
-    void registerSettings(Settings&) {}
+    void registerSettings(Settings& settings)
+    {
+       settings.registerOnOff("torsionTorque", torsion_torque, false);
+    }
+
+    inline void postSettings(IContactHistorySetup * hsetup, ContactModelBase *cmb)
+    {}
 
     void connectToProperties(PropertyRegistry & registry)
     {
@@ -88,11 +93,15 @@ namespace ContactModels
 
       const double radi = sidata.radi;
       const double radj = sidata.radj;
-      double reff=sidata.is_wall ? sidata.radi : (radi*radj/(radi+radj));
+      double reff=sidata.is_wall ? radi : (radi*radj/(radi+radj));
 
 #ifdef SUPERQUADRIC_ACTIVE_FLAG
-      if(sidata.is_non_spherical)
-        reff = MathExtraLiggghtsNonspherical::get_effective_radius(sidata);
+      if(sidata.is_non_spherical) {
+        if(sidata.is_wall)
+          reff = MathExtraLiggghtsNonspherical::get_effective_radius_wall(sidata, atom->roundness[sidata.i], error);
+        else
+          reff = MathExtraLiggghtsNonspherical::get_effective_radius(sidata, atom->roundness[sidata.i], atom->roundness[sidata.j], error);
+      }
 #endif
 
       if(sidata.is_wall){
@@ -113,12 +122,15 @@ namespace ContactModels
           r_torque[2] = rmu*Fn*wr3/wrmag*reff;
 
           // remove normal (torsion) part of torque
-          double rtorque_dot_delta = r_torque[0]*enx+ r_torque[1]*eny + r_torque[2]*enz;
-          double r_torque_n[3];
-          r_torque_n[0] = enx * rtorque_dot_delta;
-          r_torque_n[1] = eny * rtorque_dot_delta;
-          r_torque_n[2] = enz * rtorque_dot_delta;
-          vectorSubtract3D(r_torque,r_torque_n,r_torque);
+          if(!torsion_torque)
+          {
+              double rtorque_dot_delta = r_torque[0]*enx+ r_torque[1]*eny + r_torque[2]*enz;
+              double r_torque_n[3];
+              r_torque_n[0] = enx * rtorque_dot_delta;
+              r_torque_n[1] = eny * rtorque_dot_delta;
+              r_torque_n[2] = enz * rtorque_dot_delta;
+              vectorSubtract3D(r_torque,r_torque_n,r_torque);
+          }
         }
       } else {
 
@@ -135,12 +147,15 @@ namespace ContactModels
           vectorScalarMult3D(wr_roll,rmu*sidata.kn*sidata.deltan*reff/wr_rollmag,r_torque);
 
           // remove normal (torsion) part of torque
-          const double rtorque_dot_delta = r_torque[0]*enx + r_torque[1]*eny + r_torque[2]*enz;
-          double r_torque_n[3];
-          r_torque_n[0] = enx * rtorque_dot_delta;
-          r_torque_n[1] = eny * rtorque_dot_delta;
-          r_torque_n[2] = enz * rtorque_dot_delta;
-          vectorSubtract3D(r_torque,r_torque_n,r_torque);
+          if(!torsion_torque)
+          {
+              const double rtorque_dot_delta = r_torque[0]*enx + r_torque[1]*eny + r_torque[2]*enz;
+              double r_torque_n[3];
+              r_torque_n[0] = enx * rtorque_dot_delta;
+              r_torque_n[1] = eny * rtorque_dot_delta;
+              r_torque_n[2] = enz * rtorque_dot_delta;
+              vectorSubtract3D(r_torque,r_torque_n,r_torque);
+          }
         }
       }
 
@@ -159,6 +174,7 @@ namespace ContactModels
 
   private:
     double ** coeffRollFrict;
+    bool torsion_torque;
   };
 }
 }
