@@ -47,7 +47,7 @@
    Contributing author:  Axel Kohlmeyer (Temple U)
 ------------------------------------------------------------------------- */
 
-#include "string.h"
+#include <string.h>
 #include "write_dump.h"
 #include "style_dump.h"
 #include "dump.h"
